@@ -43,6 +43,7 @@ import java.util.*;
 import rice.environment.Environment;
 import rice.environment.params.simple.SimpleParameters;
 import rice.environment.random.RandomSource;
+import rice.environment.time.simulated.DirectTimeSource;
 import rice.p2p.commonapi.*;
 import rice.p2p.commonapi.Id;
 import rice.p2p.commonapi.NodeHandle;
@@ -58,7 +59,7 @@ import rice.pastry.standard.RandomNodeIdFactory;
  * commonapi. Currently is written to use Pastry nodes, but this will be
  * abstracted away.
  *
- * @version $Id: MultiringRegrTest.java 2896 2006-01-09 17:11:50Z jeffh $
+ * @version $Id: MultiringRegrTest.java 3040 2006-02-07 12:57:36Z jeffh $
  * @author Alan Mislove
  */
 public class MultiringRegrTest {
@@ -241,13 +242,13 @@ public class MultiringRegrTest {
    */
   public MultiringRegrTest(Environment env) throws IOException {
     this.environment = env;
-    if (SIMULATOR == SIMULATOR_SPHERE) {
-      simulator = new SphereNetwork(env);
-    } else {
-      simulator = new EuclideanNetwork(env);
-    }
 
     if (PROTOCOL == PROTOCOL_DIRECT) {
+      if (SIMULATOR == SIMULATOR_SPHERE) {
+        simulator = new SphereNetwork(env);
+      } else {
+        simulator = new EuclideanNetwork(env);
+      }
       factory = new DirectPastryNodeFactory(new RandomNodeIdFactory(environment), simulator, env);
     } else {
       factory = DistPastryNodeFactory.getFactory(new RandomNodeIdFactory(environment),
@@ -342,12 +343,15 @@ public class MultiringRegrTest {
    * passing.
    */
   protected void simulate() {
-    if (PROTOCOL == PROTOCOL_DIRECT) {
-      while (simulator.simulate()) {
-      }
-    } else {
-      pause(500);
+    try {
+      Thread.sleep(300);
+    } catch (InterruptedException ie) {
     }
+//    if (PROTOCOL == PROTOCOL_DIRECT) {
+//      while (simulator.simulate()) {}
+//    } else {
+//      pause(500);
+//    }
   }
 
   /**
@@ -358,11 +362,28 @@ public class MultiringRegrTest {
    * @return The created node
    */
   protected MultiringNode createNode(Id ringId, MultiringNode bootstrap) {
+    MultiringNode mn;
+
     if (bootstrap == null) {
-      return new MultiringNode(ringId, factory.newNode(null));
+      mn = new MultiringNode(ringId, factory.newNode(null));
     } else {
-      return new MultiringNode(ringId, factory.newNode(getBootstrap(bootstrap.getNode())));
+      mn = new MultiringNode(ringId, factory.newNode(getBootstrap(bootstrap.getNode())));
     }
+
+    PastryNode pn = (PastryNode) mn.getNode();
+    synchronized (pn) {
+      while (!pn.isReady()) {
+        try {
+          pn.wait(500);
+        } catch (InterruptedException ie) {
+          return null;
+        }
+      }
+      if (!pn.isReady()) {
+        System.out.println("Still waiting for node " + pn + " in ring " + ringId + " to be ready.");
+      }
+    }
+    return mn;
   }
 
   /**
@@ -706,9 +727,15 @@ public class MultiringRegrTest {
    */
   public static void main(String[] args) throws IOException {
     parseArgs(args);
-    MultiringRegrTest test = new MultiringRegrTest(new Environment(null, null, null, new DirectTimeSource(System.currentTimeMillis()), null,
-      new SimpleParameters(Environment.defaultParamFileArray, null)));
+    Environment env;
+    if (PROTOCOL == PROTOCOL_DIRECT) {
+      env = Environment.directEnvironment();
+    } else {
+      env = new Environment();
+    }
+    MultiringRegrTest test = new MultiringRegrTest(env);
     test.start();
+    env.destroy();
   }
 
   /**
