@@ -55,7 +55,7 @@ import rice.persistence.*;
 /**
  * @(#) PastImpl.java This is an implementation of the Past interface.
  *
- * @version $Id: PastImpl.java 2866 2005-12-20 15:57:54Z jstewart $
+ * @version $Id: PastImpl.java 3038 2006-02-07 10:01:01Z jeffh $
  * @author Alan Mislove
  * @author Ansley Post
  * @author Peter Druschel
@@ -427,7 +427,7 @@ public class PastImpl implements Past, Application, ReplicationManagerClient {
    */
   protected void sendRequest(Id id, PastMessage message, NodeHandle hint, Continuation command) {
     if (logger.level <= Logger.FINER) {
-      logger.log("Sending request message " + message + " to id " + id + " via " + hint);
+      logger.log("Sending request message " + message + " {" + message.getUID() + "} to id " + id + " via " + hint);
     }
     CancellableTask timer = endpoint.scheduleMessage(new MessageLostMessage(message.getUID(), getLocalNodeHandle(), id, message, hint), MESSAGE_TIMEOUT);
     insertPending(message.getUID(), timer, command);
