@@ -38,13 +38,14 @@ package rice.p2p.past.gc;
 
 import java.util.*;
 
+import rice.environment.random.RandomSource;
 import rice.p2p.commonapi.*;
 
 /**
  * @(#) GCIdFactory.java This class provides the ability to build Ids which can
  * support a multi-ring hierarchy.
  *
- * @version $Id: GCIdFactory.java,v 1.7 2005/03/11 00:58:13 jeffh Exp $
+ * @version $Id: GCIdFactory.java 2551 2005-06-06 19:01:02Z jeffh $
  * @author Alan Mislove
  * @author Peter Druschel
  */
@@ -124,6 +125,16 @@ public class GCIdFactory implements IdFactory {
     throw new UnsupportedOperationException("GCIdFactory cannot be used to build Ids!");
   }
 
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param rng DESCRIBE THE PARAMETER
+   * @return DESCRIBE THE RETURN VALUE
+   */
+  public Id buildRandomId(RandomSource rng) {
+    throw new UnsupportedOperationException("GCIdFactory cannot be used to build Ids!");
+  }
+
   /**
    * Builds an Id by converting the given toString() output back to an Id.
    * Should not normall be used.
@@ -213,7 +224,7 @@ public class GCIdFactory implements IdFactory {
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected class GCSortedMap implements SortedMap {
@@ -424,7 +435,7 @@ public class GCIdFactory implements IdFactory {
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected class GCEntrySet implements Set {
@@ -603,7 +614,7 @@ public class GCIdFactory implements IdFactory {
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
   protected class GCMapEntry implements Map.Entry {
