@@ -37,21 +37,33 @@ if advised of the possibility of such damage.
 package rice.p2p.scribe;
 
 import java.util.*;
-import java.util.logging.*;
 import java.util.prefs.*;
 
 import rice.*;
+import rice.environment.Environment;
+import rice.environment.logging.Logger;
+import rice.environment.params.Parameters;
 import rice.p2p.commonapi.*;
 import rice.p2p.scribe.messaging.*;
 
 /**
  * @(#) ScribeImpl.java Thie provided implementation of Scribe.
  *
- * @version $Id: ScribeImpl.java,v 1.26 2005/05/16 17:46:55 amislove Exp $
+ * @version $Id: ScribeImpl.java 2885 2006-01-05 12:20:54Z jeffh $
  * @author Alan Mislove
  */
 public class ScribeImpl implements Scribe, Application {
 
+  /**
+   * The interval with which to perform maintenance
+   */
+  public final int MAINTENANCE_INTERVAL;
+
+  /**
+   * the timeout for a subscribe message
+   */
+  public final int MESSAGE_TIMEOUT;
+
   /**
    * the hashtable of topic -> TopicManager
    */
@@ -67,11 +79,6 @@ public class ScribeImpl implements Scribe, Application {
    */
   protected Endpoint endpoint;
 
-  /**
-   * the logger which we will use
-   */
-  protected Logger log = Logger.getLogger(this.getClass().getName());
-
   /**
    * the local node handle
    */
@@ -92,15 +99,11 @@ public class ScribeImpl implements Scribe, Application {
    */
   private int id;
 
-  /**
-   * The interval with which to perform maintenance
-   */
-  public static int MAINTENANCE_INTERVAL = 180000;
+  Environment environment;
 
-  /**
-   * the timeout for a subscribe message
-   */
-  public static int MESSAGE_TIMEOUT = 15000;
+  Logger logger;
+
+  private String instance;
 
   /**
    * Constructor for Scribe, using the default policy.
@@ -109,7 +112,7 @@ public class ScribeImpl implements Scribe, Application {
    * @param instance The unique instance name of this Scribe
    */
   public ScribeImpl(Node node, String instance) {
-    this(node, new ScribePolicy.DefaultScribePolicy(), instance);
+    this(node, new ScribePolicy.DefaultScribePolicy(node.getEnvironment()), instance);
   }
 
   /**
@@ -120,6 +123,13 @@ public class ScribeImpl implements Scribe, Application {
    * @param instance The unique instance name of this Scribe
    */
   public ScribeImpl(Node node, ScribePolicy policy, String instance) {
+    this.environment = node.getEnvironment();
+    logger = environment.getLogManager().getLogger(ScribeImpl.class, instance);
+
+    Parameters p = environment.getParameters();
+    MAINTENANCE_INTERVAL = p.getInt("p2p_scribe_maintenance_interval");
+    MESSAGE_TIMEOUT = p.getInt("p2p_scribe_message_timeout");
+    this.instance = instance;
     this.endpoint = node.registerApplication(this, instance);
     this.topics = new Hashtable();
     this.outstanding = new Hashtable();
@@ -129,13 +139,20 @@ public class ScribeImpl implements Scribe, Application {
     this.id = Integer.MIN_VALUE;
 
     // schedule the period liveness checks of the parent
-    endpoint.scheduleMessage(new MaintenanceMessage(), (new Random()).nextInt(MAINTENANCE_INTERVAL), MAINTENANCE_INTERVAL);
+    endpoint.scheduleMessage(new MaintenanceMessage(), environment.getRandomSource().nextInt(MAINTENANCE_INTERVAL), MAINTENANCE_INTERVAL);
 
-    // log.addHandler(new ConsoleHandler());
-    // log.setLevel(Level.FINEST);
-    // log.getHandlers()[0].setLevel(Level.FINEST);
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Starting up Scribe");
+    }
+  }
 
-    log.finer(endpoint.getId() + ": Starting up Scribe");
+  /**
+   * Gets the Environment attribute of the ScribeImpl object
+   *
+   * @return The Environment value
+   */
+  public Environment getEnvironment() {
+    return environment;
   }
 
   /**
@@ -266,7 +283,18 @@ public class ScribeImpl implements Scribe, Application {
   private void sendSubscribe(Topic topic, ScribeClient client, ScribeContent content, Id previousParent) {
     id++;
 
-    log.finest(endpoint.getId() + ": Sending subscribe message for topic " + topic);
+    if (logger.level <= Logger.FINEST) {
+      logger.log(endpoint.getId() + ": Sending subscribe message for topic " + topic + " client:" + client);
+    }
+    //logException(Logger.FINEST,"Stack Trace",new Exception("StackTrace"));
+
+    // TODO: This should go to the ScribeImpl
+    if (client == null) {
+      ScribeClient[] clients = getClients(topic);
+      if (clients.length > 0) {
+        client = clients[0];
+      }
+    }
 
     if (client != null) {
       outstanding.put(new Integer(id), client);
@@ -285,7 +313,9 @@ public class ScribeImpl implements Scribe, Application {
    */
   private void ackMessageReceived(SubscribeAckMessage message) {
     ScribeClient client = (ScribeClient) outstanding.remove(new Integer(message.getId()));
-    log.finer(endpoint.getId() + ": Removing client " + client + " from list of outstanding for ack " + message.getId());
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Removing client " + client + " from list of outstanding for ack " + message.getId());
+    }
     CancellableTask task = (CancellableTask) lost.remove(new Integer(message.getId()));
 
     if (task != null) {
@@ -302,7 +332,9 @@ public class ScribeImpl implements Scribe, Application {
     ScribeClient client = (ScribeClient) outstanding.remove(new Integer(message.getId()));
     lost.remove(new Integer(message.getId()));
 
-    log.finer(endpoint.getId() + ": Telling client " + client + " about FAILURE for outstanding ack " + message.getId());
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Telling client " + client + " about FAILURE for outstanding ack " + message.getId());
+    }
 
     if (client != null) {
       client.subscribeFailed(message.getTopic());
@@ -318,7 +350,9 @@ public class ScribeImpl implements Scribe, Application {
     ScribeClient client = (ScribeClient) outstanding.remove(new Integer(message.getId()));
     lost.remove(new Integer(message.getId()));
 
-    log.finer(endpoint.getId() + ": Telling client " + client + " about LOSS for outstanding ack " + message.getId());
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Telling client " + client + " about LOSS for outstanding ack " + message.getId());
+    }
 
     if (client != null) {
       client.subscribeFailed(message.getTopic());
@@ -347,7 +381,9 @@ public class ScribeImpl implements Scribe, Application {
    * @param content DESCRIBE THE PARAMETER
    */
   public void subscribe(Topic topic, ScribeClient client, ScribeContent content) {
-    log.finer(endpoint.getId() + ": Subscribing client " + client + " to topic " + topic);
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Subscribing client " + client + " to topic " + topic);
+    }
 
     // if we don't know about this topic, subscribe
     // otherwise, we simply add the client to the list
@@ -372,7 +408,9 @@ public class ScribeImpl implements Scribe, Application {
    * @param client The client to unsubscribe
    */
   public void unsubscribe(Topic topic, ScribeClient client) {
-    log.finer(endpoint.getId() + ": Unsubscribing client " + client + " from topic " + topic);
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Unsubscribing client " + client + " from topic " + topic);
+    }
 
     if (topics.get(topic) != null) {
       TopicManager manager = (TopicManager) topics.get(topic);
@@ -388,7 +426,9 @@ public class ScribeImpl implements Scribe, Application {
         }
       }
     } else {
-      log.warning(endpoint.getId() + ": Attempt to unsubscribe client " + client + " from unknown topic " + topic);
+      if (logger.level <= Logger.WARNING) {
+        logger.log(endpoint.getId() + ": Attempt to unsubscribe client " + client + " from unknown topic " + topic);
+      }
     }
   }
 
@@ -399,7 +439,9 @@ public class ScribeImpl implements Scribe, Application {
    * @param content The content to publish
    */
   public void publish(Topic topic, ScribeContent content) {
-    log.finer(endpoint.getId() + ": Publishing content " + content + " to topic " + topic);
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Publishing content " + content + " to topic " + topic);
+    }
 
     endpoint.route(topic.getId(), new PublishRequestMessage(handle, topic, content), null);
   }
@@ -411,7 +453,9 @@ public class ScribeImpl implements Scribe, Application {
    * @param content The content to anycast
    */
   public void anycast(Topic topic, ScribeContent content) {
-    log.finer(endpoint.getId() + ": Anycasting content " + content + " to topic " + topic);
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Anycasting content " + content + " to topic " + topic);
+    }
 
     endpoint.route(topic.getId(), new AnycastMessage(handle, topic, content), null);
   }
@@ -435,7 +479,9 @@ public class ScribeImpl implements Scribe, Application {
    * @param id THe seuqnce number
    */
   protected void addChild(Topic topic, NodeHandle child, int id) {
-    log.finer(endpoint.getId() + ": Adding child " + child + " to topic " + topic);
+    if (logger.level <= Logger.FINER) {
+      logger.log(endpoint.getId() + ": Adding child " + child + " to topic " + topic);
+    }
     TopicManager manager = (TopicManager) topics.get(topic);
 
     // if we don't know about the topic, we subscribe, otherwise,
@@ -444,7 +490,9 @@ public class ScribeImpl implements Scribe, Application {
       manager = new TopicManager(topic, child);
       topics.put(topic, manager);
 
-      log.finer(endpoint.getId() + ": Implicitly subscribing to topic " + topic);
+      if (logger.level <= Logger.FINER) {
+        logger.log(endpoint.getId() + ": Implicitly subscribing to topic " + topic);
+      }
       sendSubscribe(topic, null, null);
     } else {
       manager.addChild(child);
@@ -481,7 +529,9 @@ public class ScribeImpl implements Scribe, Application {
    * @param sendDrop Whether or not to send a drop message to the chil
    */
   protected void removeChild(Topic topic, NodeHandle child, boolean sendDrop) {
-    log.fine(endpoint.getId() + ": Removing child " + child + " from topic " + topic);
+    if (logger.level <= Logger.FINE) {
+      logger.log(endpoint.getId() + ": Removing child " + child + " from topic " + topic);
+    }
 
     if (topics.get(topic) != null) {
       TopicManager manager = (TopicManager) topics.get(topic);
@@ -491,7 +541,9 @@ public class ScribeImpl implements Scribe, Application {
       if (manager.removeChild(child)) {
         topics.remove(topic);
         NodeHandle parent = manager.getParent();
-        log.fine(endpoint.getId() + ": We no longer need topic " + topic + " - unsubscribing from parent " + parent);
+        if (logger.level <= Logger.FINE) {
+          logger.log(endpoint.getId() + ": We no longer need topic " + topic + " - unsubscribing from parent " + parent);
+        }
 
         if (parent != null) {
           endpoint.route(null, new UnsubscribeMessage(handle, topic), parent);
@@ -499,7 +551,9 @@ public class ScribeImpl implements Scribe, Application {
       }
 
       if ((sendDrop) && (child.isAlive())) {
-        log.fine(endpoint.getId() + ": Informing child " + child + " that he has been dropped from topic " + topic);
+        if (logger.level <= Logger.FINE) {
+          logger.log(endpoint.getId() + ": Informing child " + child + " that he has been dropped from topic " + topic);
+        }
 
         // now, we tell the child that he has been dropped
         endpoint.route(null, new DropMessage(handle, topic), child);
@@ -514,7 +568,9 @@ public class ScribeImpl implements Scribe, Application {
         clients[i].childRemoved(topic, child);
       }
     } else {
-      log.warning(endpoint.getId() + ": Unexpected attempt to remove child " + child + " from unknown topic " + topic);
+      if (logger.level <= Logger.WARNING) {
+        logger.log(endpoint.getId() + ": Unexpected attempt to remove child " + child + " from unknown topic " + topic);
+      }
     }
   }
 
@@ -533,7 +589,9 @@ public class ScribeImpl implements Scribe, Application {
    * @return Whether or not to forward the message further
    */
   public boolean forward(final RouteMessage message) {
-    log.finest(endpoint.getId() + ": Forward called with " + message.getMessage());
+    if (logger.level <= Logger.FINEST) {
+      logger.log(endpoint.getId() + ": Forward called with " + message.getMessage());
+    }
 
     if (message.getMessage() instanceof AnycastMessage) {
       AnycastMessage aMessage = (AnycastMessage) message.getMessage();
@@ -557,9 +615,11 @@ public class ScribeImpl implements Scribe, Application {
           List path = Arrays.asList(manager.getPathToRoot());
 
           if (path.contains(previousParent)) {
-            log.info(endpoint.getId() + ": Rejecting subscribe message from " +
-              sMessage.getSubscriber() + " for topic " + sMessage.getTopic() +
-              " because we are on the subscriber's path to the root.");
+            if (logger.level <= Logger.INFO) {
+              logger.log(endpoint.getId() + ": Rejecting subscribe message from " +
+                sMessage.getSubscriber() + " for topic " + sMessage.getTopic() +
+                " because we are on the subscriber's path to the root.");
+            }
             return true;
           }
         }
@@ -579,8 +639,10 @@ public class ScribeImpl implements Scribe, Application {
 
         // see if the policy will allow us to take on this child
         if (policy.allowSubscribe(sMessage, clients, handles)) {
-          log.finer(endpoint.getId() + ": Hijacking subscribe message from " +
-            sMessage.getSubscriber() + " for topic " + sMessage.getTopic());
+          if (logger.level <= Logger.FINER) {
+            logger.log(endpoint.getId() + ": Hijacking subscribe message from " +
+              sMessage.getSubscriber() + " for topic " + sMessage.getTopic());
+          }
 
           // if so, add the child
           addChild(sMessage.getTopic(), sMessage.getSubscriber(), sMessage.getId());
@@ -588,8 +650,10 @@ public class ScribeImpl implements Scribe, Application {
         }
 
         // otherwise, we are effectively rejecting the child
-        log.finer(endpoint.getId() + ": Rejecting subscribe message from " +
-          sMessage.getSubscriber() + " for topic " + sMessage.getTopic());
+        if (logger.level <= Logger.FINER) {
+          logger.log(endpoint.getId() + ": Rejecting subscribe message from " +
+            sMessage.getSubscriber() + " for topic " + sMessage.getTopic());
+        }
 
         // if we are not associated with this topic at all, we simply let the subscribe go
         // closer to the root
@@ -608,8 +672,10 @@ public class ScribeImpl implements Scribe, Application {
         // see if one of our clients will accept the anycast
         for (int i = 0; i < clients.length; i++) {
           if (clients[i].anycast(aMessage.getTopic(), aMessage.getContent())) {
-            log.finer(endpoint.getId() + ": Accepting anycast message from " +
-              aMessage.getSource() + " for topic " + aMessage.getTopic());
+            if (logger.level <= Logger.FINER) {
+              logger.log(endpoint.getId() + ": Accepting anycast message from " +
+                aMessage.getSource() + " for topic " + aMessage.getTopic());
+            }
 
             return false;
           }
@@ -623,8 +689,10 @@ public class ScribeImpl implements Scribe, Application {
           return true;
         }
 
-        log.finer(endpoint.getId() + ": Rejecting anycast message from " +
-          aMessage.getSource() + " for topic " + aMessage.getTopic());
+        if (logger.level <= Logger.FINER) {
+          logger.log(endpoint.getId() + ": Rejecting anycast message from " +
+            aMessage.getSource() + " for topic " + aMessage.getTopic());
+        }
       }
 
       // add the local node to the visited list
@@ -644,16 +712,22 @@ public class ScribeImpl implements Scribe, Application {
         handle = aMessage.getNext();
       }
 
-      log.finer(endpoint.getId() + ": Forwarding anycast message for topic " + aMessage.getTopic() + "on to " + handle);
+      if (logger.level <= Logger.FINER) {
+        logger.log(endpoint.getId() + ": Forwarding anycast message for topic " + aMessage.getTopic() + "on to " + handle);
+      }
 
       if (handle == null) {
-        log.fine(endpoint.getId() + ": Anycast " + aMessage + " failed.");
+        if (logger.level <= Logger.FINE) {
+          logger.log(endpoint.getId() + ": Anycast " + aMessage + " failed.");
+        }
 
         // if it's a subscribe message, send a subscribe failed message back
         // as a courtesy
         if (aMessage instanceof SubscribeMessage) {
           SubscribeMessage sMessage = (SubscribeMessage) aMessage;
-          log.finer(endpoint.getId() + ": Sending SubscribeFailedMessage to " + sMessage.getSubscriber());
+          if (logger.level <= Logger.FINER) {
+            logger.log(endpoint.getId() + ": Sending SubscribeFailedMessage to " + sMessage.getSubscriber());
+          }
 
           endpoint.route(null,
             new SubscribeFailedMessage(handle, sMessage.getTopic(), sMessage.getId()),
@@ -677,7 +751,9 @@ public class ScribeImpl implements Scribe, Application {
    * @param message The message being sent
    */
   public void deliver(Id id, Message message) {
-    log.finest(endpoint.getId() + ": Deliver called with " + id + " " + message);
+    if (logger.level <= Logger.FINEST) {
+      logger.log(endpoint.getId() + ": Deliver called with " + id + " " + message);
+    }
 
     if (message instanceof AnycastMessage) {
       AnycastMessage aMessage = (AnycastMessage) message;
@@ -691,25 +767,33 @@ public class ScribeImpl implements Scribe, Application {
           SubscribeMessage sMessage = (SubscribeMessage) message;
 
           outstanding.remove(new Integer(sMessage.getId()));
-          log.fine(endpoint.getId() + ": Received our own subscribe message " + aMessage + " for topic " +
-            aMessage.getTopic() + " - we are the root.");
+          if (logger.level <= Logger.FINE) {
+            logger.log(endpoint.getId() + ": Received our own subscribe message " + aMessage + " for topic " +
+              aMessage.getTopic() + " - we are the root.");
+          }
         } else {
-          log.warning(endpoint.getId() + ": Received unexpected delivered anycast message " + aMessage + " for topic " +
-            aMessage.getTopic() + " - was generated by us.");
+          if (logger.level <= Logger.WARNING) {
+            logger.log(endpoint.getId() + ": Received unexpected delivered anycast message " + aMessage + " for topic " +
+              aMessage.getTopic() + " - was generated by us.");
+          }
         }
       } else {
         // here, we have had a subscribe message delivered, which means that we are the root, but
         // our policy says that we cannot take on this child
         if (aMessage instanceof SubscribeMessage) {
           SubscribeMessage sMessage = (SubscribeMessage) aMessage;
-          log.fine(endpoint.getId() + ": Sending SubscribeFailedMessage (at root) to " + sMessage.getSubscriber());
+          if (logger.level <= Logger.FINE) {
+            logger.log(endpoint.getId() + ": Sending SubscribeFailedMessage (at root) to " + sMessage.getSubscriber());
+          }
 
           endpoint.route(null,
             new SubscribeFailedMessage(handle, sMessage.getTopic(), sMessage.getId()),
             sMessage.getSubscriber());
         } else {
-          log.warning(endpoint.getId() + ": Received unexpected delivered anycast message " + aMessage + " for topic " +
-            aMessage.getTopic() + " - not generated by us, but was expected to be.");
+          if (logger.level <= Logger.WARNING) {
+            logger.log(endpoint.getId() + ": Received unexpected delivered anycast message " + aMessage + " for topic " +
+              aMessage.getTopic() + " - not generated by us, but was expected to be.");
+          }
         }
       }
     } else if (message instanceof SubscribeAckMessage) {
@@ -718,16 +802,22 @@ public class ScribeImpl implements Scribe, Application {
 
       ackMessageReceived(saMessage);
 
-      log.fine(endpoint.getId() + ": Received subscribe ack message from " + saMessage.getSource() + " for topic " + saMessage.getTopic());
+      if (logger.level <= Logger.FINE) {
+        logger.log(endpoint.getId() + ": Received subscribe ack message from " + saMessage.getSource() + " for topic " + saMessage.getTopic());
+      }
 
       if (!saMessage.getSource().isAlive()) {
-        log.warning(endpoint.getId() + ": Received subscribe ack message from " + saMessage.getSource() + " for topic " + saMessage.getTopic());
+        if (logger.level <= Logger.WARNING) {
+          logger.log(endpoint.getId() + ": Received subscribe ack message from " + saMessage.getSource() + " for topic " + saMessage.getTopic());
+        }
       }
 
       // if we're the root, reject the ack message
       if (isRoot(saMessage.getTopic())) {
-        log.fine(endpoint.getId() + ": Received unexpected subscribe ack message (we are the root) from " +
-          saMessage.getSource() + " for topic " + saMessage.getTopic());
+        if (logger.level <= Logger.FINE) {
+          logger.log(endpoint.getId() + ": Received unexpected subscribe ack message (we are the root) from " +
+            saMessage.getSource() + " for topic " + saMessage.getTopic());
+        }
         endpoint.route(null, new UnsubscribeMessage(handle, saMessage.getTopic()), saMessage.getSource());
       } else {
         // if we don't know about this topic, then we unsubscribe
@@ -741,8 +831,10 @@ public class ScribeImpl implements Scribe, Application {
           if (manager.getParent().equals(saMessage.getSource())) {
             manager.setPathToRoot(saMessage.getPathToRoot());
           } else {
-            log.warning(endpoint.getId() + ": Received somewhat unexpected subscribe ack message (already have parent " + manager.getParent() +
-              ") from " + saMessage.getSource() + " for topic " + saMessage.getTopic() + " - the new policy is now to accept the message");
+            if (logger.level <= Logger.WARNING) {
+              logger.log(endpoint.getId() + ": Received somewhat unexpected subscribe ack message (already have parent " + manager.getParent() +
+                ") from " + saMessage.getSource() + " for topic " + saMessage.getTopic() + " - the new policy is now to accept the message");
+            }
 
             NodeHandle parent = manager.getParent();
             manager.setParent(saMessage.getSource());
@@ -751,8 +843,10 @@ public class ScribeImpl implements Scribe, Application {
             endpoint.route(null, new UnsubscribeMessage(handle, saMessage.getTopic()), parent);
           }
         } else {
-          log.warning(endpoint.getId() + ": Received unexpected subscribe ack message from " +
-            saMessage.getSource() + " for unknown topic " + saMessage.getTopic());
+          if (logger.level <= Logger.WARNING) {
+            logger.log(endpoint.getId() + ": Received unexpected subscribe ack message from " +
+              saMessage.getSource() + " for unknown topic " + saMessage.getTopic());
+          }
 
           endpoint.route(null, new UnsubscribeMessage(handle, saMessage.getTopic()), saMessage.getSource());
         }
@@ -769,14 +863,18 @@ public class ScribeImpl implements Scribe, Application {
       PublishRequestMessage prMessage = (PublishRequestMessage) message;
       TopicManager manager = (TopicManager) topics.get(prMessage.getTopic());
 
-      log.finer(endpoint.getId() + ": Received publish request message with data " +
-        prMessage.getContent() + " for topic " + prMessage.getTopic());
+      if (logger.level <= Logger.FINER) {
+        logger.log(endpoint.getId() + ": Received publish request message with data " +
+          prMessage.getContent() + " for topic " + prMessage.getTopic());
+      }
 
       // if message is for a non-existant topic, drop it on the floor (we are the root, after all)
       // otherwise, turn it into a publish message, and forward it on
       if (manager == null) {
-        log.fine(endpoint.getId() + ": Received publish request message for non-existent topic " +
-          prMessage.getTopic() + " - dropping on floor.");
+        if (logger.level <= Logger.FINE) {
+          logger.log(endpoint.getId() + ": Received publish request message for non-existent topic " +
+            prMessage.getTopic() + " - dropping on floor.");
+        }
       } else {
         deliver(prMessage.getTopic().getId(), new PublishMessage(prMessage.getSource(), prMessage.getTopic(), prMessage.getContent()));
       }
@@ -784,7 +882,9 @@ public class ScribeImpl implements Scribe, Application {
       PublishMessage pMessage = (PublishMessage) message;
       TopicManager manager = (TopicManager) topics.get(pMessage.getTopic());
 
-      log.finer(endpoint.getId() + ": Received publish message with data " + pMessage.getContent() + " for topic " + pMessage.getTopic());
+      if (logger.level <= Logger.FINER) {
+        logger.log(endpoint.getId() + ": Received publish message with data " + pMessage.getContent() + " for topic " + pMessage.getTopic());
+      }
 
       // if we don't know about this topic, or this message did
       // not come from our parent, send an unsubscribe message
@@ -796,33 +896,43 @@ public class ScribeImpl implements Scribe, Application {
         ScribeClient[] clients = manager.getClients();
 
         for (int i = 0; i < clients.length; i++) {
-          log.finer(endpoint.getId() + ": Delivering publish message with data " + pMessage.getContent() + " for topic " +
-            pMessage.getTopic() + " to client " + clients[i]);
+          if (logger.level <= Logger.FINER) {
+            logger.log(endpoint.getId() + ": Delivering publish message with data " + pMessage.getContent() + " for topic " +
+              pMessage.getTopic() + " to client " + clients[i]);
+          }
           clients[i].deliver(pMessage.getTopic(), pMessage.getContent());
         }
 
         NodeHandle[] handles = manager.getChildren();
 
         for (int i = 0; i < handles.length; i++) {
-          log.finer(endpoint.getId() + ": Forwarding publish message with data " + pMessage.getContent() + " for topic " +
-            pMessage.getTopic() + " to child " + handles[i]);
+          if (logger.level <= Logger.FINER) {
+            logger.log(endpoint.getId() + ": Forwarding publish message with data " + pMessage.getContent() + " for topic " +
+              pMessage.getTopic() + " to child " + handles[i]);
+          }
           endpoint.route(null, new PublishMessage(endpoint.getLocalNodeHandle(), pMessage.getTopic(), pMessage.getContent()), handles[i]);
         }
       } else {
-        log.warning(endpoint.getId() + ": Received unexpected publish message from " +
-          pMessage.getSource() + " for unknown topic " + pMessage.getTopic());
+        if (logger.level <= Logger.WARNING) {
+          logger.log(endpoint.getId() + ": Received unexpected publish message from " +
+            pMessage.getSource() + " for unknown topic " + pMessage.getTopic());
+        }
 
         endpoint.route(null, new UnsubscribeMessage(handle, pMessage.getTopic()), pMessage.getSource());
       }
     } else if (message instanceof UnsubscribeMessage) {
       UnsubscribeMessage uMessage = (UnsubscribeMessage) message;
-      log.fine(endpoint.getId() + ": Received unsubscribe message from " +
-        uMessage.getSource() + " for topic " + uMessage.getTopic());
+      if (logger.level <= Logger.FINE) {
+        logger.log(endpoint.getId() + ": Received unsubscribe message from " +
+          uMessage.getSource() + " for topic " + uMessage.getTopic());
+      }
 
       removeChild(uMessage.getTopic(), uMessage.getSource(), false);
     } else if (message instanceof DropMessage) {
       DropMessage dMessage = (DropMessage) message;
-      log.fine(endpoint.getId() + ": Received drop message from " + dMessage.getSource() + " for topic " + dMessage.getTopic());
+      if (logger.level <= Logger.FINE) {
+        logger.log(endpoint.getId() + ": Received drop message from " + dMessage.getSource() + " for topic " + dMessage.getTopic());
+      }
 
       TopicManager manager = (TopicManager) topics.get(dMessage.getTopic());
 
@@ -838,15 +948,21 @@ public class ScribeImpl implements Scribe, Application {
             sendSubscribe(dMessage.getTopic(), null, null);
           }
         } else {
-          log.warning(endpoint.getId() + ": Received unexpected drop message from non-parent " +
-            dMessage.getSource() + " for topic " + dMessage.getTopic() + " - ignoring");
+          if (logger.level <= Logger.WARNING) {
+            logger.log(endpoint.getId() + ": Received unexpected drop message from non-parent " +
+              dMessage.getSource() + " for topic " + dMessage.getTopic() + " - ignoring");
+          }
         }
       } else {
-        log.warning(endpoint.getId() + ": Received unexpected drop message from " +
-          dMessage.getSource() + " for unknown topic " + dMessage.getTopic() + " - ignoring");
+        if (logger.level <= Logger.WARNING) {
+          logger.log(endpoint.getId() + ": Received unexpected drop message from " +
+            dMessage.getSource() + " for unknown topic " + dMessage.getTopic() + " - ignoring");
+        }
       }
     } else if (message instanceof MaintenanceMessage) {
-      log.fine(endpoint.getId() + ": Received maintenance message");
+      if (logger.level <= Logger.FINE) {
+        logger.log(endpoint.getId() + ": Received maintenance message");
+      }
 
       Iterator i = topics.values().iterator();
 
@@ -862,7 +978,9 @@ public class ScribeImpl implements Scribe, Application {
         }
       }
     } else {
-      log.warning(endpoint.getId() + ": Received unknown message " + message + " - dropping on floor.");
+      if (logger.level <= Logger.WARNING) {
+        logger.log(endpoint.getId() + ": Received unknown message " + message + " - dropping on floor.");
+      }
     }
   }
 
@@ -876,7 +994,10 @@ public class ScribeImpl implements Scribe, Application {
    */
   public void update(NodeHandle handle, boolean joined) {
     Set set = topics.keySet();
-    Iterator e = set.iterator();
+    Iterator e;
+    synchronized (set) {
+      e = new ArrayList(set).iterator();
+    }
     TopicManager manager;
     Topic topic;
 
@@ -899,13 +1020,24 @@ public class ScribeImpl implements Scribe, Application {
     }
   }
 
+  /**
+   * DESCRIBE THE METHOD
+   */
+  public void destroy() {
+    Iterator topicIter = topics.values().iterator();
+    while (topicIter.hasNext()) {
+      TopicManager topicManager = (TopicManager) topicIter.next();
+      topicManager.destroy();
+    }
+  }
+
   /**
    * Class which keeps track of a given topic
    *
-   * @version $Id: ScribeImpl.java,v 1.26 2005/05/16 17:46:55 amislove Exp $
+   * @version $Id: ScribeImpl.java 2885 2006-01-05 12:20:54Z jeffh $
    * @author amislove
    */
-  public class TopicManager implements Observer {
+  public class TopicManager implements Observer, Destructable {
 
     /**
      * DESCRIBE THE FIELD
@@ -1044,7 +1176,9 @@ public class ScribeImpl implements Scribe, Application {
      */
     public void setParent(NodeHandle handle) {
       if ((handle != null) && (parent != null)) {
-        log.warning(endpoint.getId() + ": Unexpectedly changing parents for topic " + topic);
+        if (logger.level <= Logger.WARNING) {
+          logger.log(endpoint.getId() + ": Unexpectedly changing parents for topic " + topic);
+        }
       }
 
       if (parent != null) {
@@ -1078,14 +1212,18 @@ public class ScribeImpl implements Scribe, Application {
     public void update(Observable o, Object arg) {
       if (arg.equals(NodeHandle.DECLARED_DEAD)) {
         if (children.contains(o)) {
-          log.fine(endpoint.getId() + ": Child " + o + " for topic " + topic + " has died - removing.");
+          if (logger.level <= Logger.FINE) {
+            logger.log(endpoint.getId() + ": Child " + o + " for topic " + topic + " has died - removing.");
+          }
 
           ScribeImpl.this.removeChild(topic, (NodeHandle) o);
         }
 
         if (o.equals(parent)) {
           // if our parent has died, then we must resubscribe to the topic
-          log.fine(endpoint.getId() + ": Parent " + parent + " for topic " + topic + " has died - resubscribing.");
+          if (logger.level <= Logger.FINE) {
+            logger.log(endpoint.getId() + ": Parent " + parent + " for topic " + topic + " has died - resubscribing.");
+          }
 
           setParent(null);
 
@@ -1095,7 +1233,9 @@ public class ScribeImpl implements Scribe, Application {
             sendSubscribe(topic, null, null, ((NodeHandle) o).getId());
           }
         } else {
-          log.warning(endpoint.getId() + ": Received unexpected update from " + o);
+          if (logger.level <= Logger.WARNING) {
+            logger.log(endpoint.getId() + ": Received unexpected update from " + o);
+          }
           o.deleteObserver(this);
         }
       }
@@ -1164,5 +1304,19 @@ public class ScribeImpl implements Scribe, Application {
 
       return unsub;
     }
+
+    /**
+     * DESCRIBE THE METHOD
+     */
+    public void destroy() {
+      if (parent != null) {
+        parent.deleteObserver(this);
+      }
+      Iterator i = children.iterator();
+      while (i.hasNext()) {
+        NodeHandle child = (NodeHandle) i.next();
+        child.deleteObserver(this);
+      }
+    }
   }
 }
