@@ -50,7 +50,7 @@ import rice.environment.random.RandomSource;
  * Ids are to be constructed from the build() methods, which ensure that only
  * one copy of each Id is in memory at a time.
  *
- * @version $Id: Id.java 2634 2005-07-07 21:18:42Z jeffh $
+ * @version $Id: Id.java 2913 2006-01-11 19:52:58Z jeffh $
  * @author Andrew Ladd
  * @author Peter Druschel
  * @author Alan Mislove
@@ -80,7 +80,7 @@ public class Id implements rice.p2p.commonapi.Id {
   /**
    * DESCRIBE THE FIELD
    */
-  protected final static int nlen = IdBitLength / 32;
+  public final static int nlen = IdBitLength / 32;
 
   /**
    * serialver for backwards compatibility
@@ -486,6 +486,26 @@ public class Id implements rice.p2p.commonapi.Id {
     return d;
   }
 
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param nid DESCRIBE THE PARAMETER
+   * @param d DESCRIBE THE PARAMETER
+   * @return DESCRIBE THE RETURN VALUE
+   */
+  public Distance distance(Id nid, Distance d) {
+    int[] dist = d.difference;
+    absDistance(nid, dist);
+
+    if ((dist[nlen - 1] & 0x80000000) != 0) {
+      invert(dist);
+    }
+
+//    Distance d = new Distance(dist);
+
+    return d;
+  }
+
   /**
    * Returns the longer numerical distance on the ring between a pair of Ids.
    *
@@ -760,7 +780,17 @@ public class Id implements rice.p2p.commonapi.Id {
    * @return an int[] containing the distance between this and nid.
    */
   private int[] absDistance(Id nid) {
-    int dist[] = new int[nlen];
+    return absDistance(nid, new int[nlen]);
+  }
+
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param nid DESCRIBE THE PARAMETER
+   * @param dist DESCRIBE THE PARAMETER
+   * @return DESCRIBE THE RETURN VALUE
+   */
+  private int[] absDistance(Id nid, int dist[]) {
     long x;
     long y;
     long diff;
@@ -797,7 +827,6 @@ public class Id implements rice.p2p.commonapi.Id {
         dist[i] = (int) diff;
       }
     }
-
     return dist;
   }
 
@@ -997,7 +1026,7 @@ public class Id implements rice.p2p.commonapi.Id {
    * A class for representing and manipulating the distance between two Ids on
    * the circle.
    *
-   * @version $Id: Id.java 2634 2005-07-07 21:18:42Z jeffh $
+   * @version $Id: Id.java 2913 2006-01-11 19:52:58Z jeffh $
    * @author amislove
    */
   public static class Distance implements rice.p2p.commonapi.Id.Distance {
