@@ -50,7 +50,7 @@ import java.util.*;
  * A PastryAppl is an abstract class that every Pastry application extends. This
  * is the external Pastry API.
  *
- * @version $Id: PastryAppl.java 2808 2005-11-22 14:38:49Z jeffh $
+ * @version $Id: PastryAppl.java 3248 2006-04-27 13:04:07Z jeffh $
  * @author Peter Druschel
  */
 public abstract class PastryAppl implements MessageReceiver {
@@ -95,32 +95,40 @@ public abstract class PastryAppl implements MessageReceiver {
    * @param instance The instance name of this appl.
    */
   public PastryAppl(PastryNode pn, String instance) {
+    this(pn, instance, null);
+    register();
+  }
+
+  /**
+   * Constructor for PastryAppl.
+   *
+   * @param pn DESCRIBE THE PARAMETER
+   * @param instance DESCRIBE THE PARAMETER
+   * @param address DESCRIBE THE PARAMETER
+   */
+  public PastryAppl(PastryNode pn, String instance, Address address) {
+    this.address = address;
     if (instance != null) {
       this.instance = instance;
-      this.address = new StandardAddress(this.getClass(), instance, pn.getEnvironment());
+      if (address == null) {
+        this.address = new StandardAddress(this.getClass(), instance, pn.getEnvironment());
+      }
     }
 
     thePastryNode = pn;
-    thePastryNode.registerReceiver(getCredentials(), getAddress(), this);
-
-    thePastryNode.addLeafSetObserver(new LeafSetObserver());
-    thePastryNode.addRouteSetObserver(new RouteSetObserver());
-
-    thePastryNode.registerApp(this);
-    // just adds it to a list
     logger = pn.getEnvironment().getLogManager().getLogger(getClass(), instance);
   }
 
   /**
    * Constructor. This constructor will perform the same tasks as the above
    * constructor, but will also create a Pastry address for this application,
-   * using the specified port.
+   * using the specified port. Need to call register on this.
    *
    * @param pn the pastry node that client will attach to.
    * @param port DESCRIBE THE PARAMETER
    */
   public PastryAppl(PastryNode pn, int port) {
-    this(pn, "[PORT " + port + "]");
+    this(pn, "[PORT " + port + "]", new StandardAddress(port));
   }
 
   /**
@@ -199,6 +207,19 @@ public abstract class PastryAppl implements MessageReceiver {
    */
   public abstract Credentials getCredentials();
 
+  /**
+   * DESCRIBE THE METHOD
+   */
+  protected void register() {
+    thePastryNode.registerReceiver(getCredentials(), getAddress(), this);
+
+    thePastryNode.addLeafSetListener(new LeafSetObserver());
+    thePastryNode.addRouteSetListener(new RouteSetObserver());
+
+    thePastryNode.registerApp(this);
+    // just adds it to a list
+  }
+
   // internal methods
 
   /**
@@ -379,20 +400,16 @@ public abstract class PastryAppl implements MessageReceiver {
    * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
-  private class LeafSetObserver implements Observer {
+  private class LeafSetObserver implements NodeSetListener {
     /**
      * DESCRIBE THE METHOD
      *
-     * @param o DESCRIBE THE PARAMETER
-     * @param arg DESCRIBE THE PARAMETER
+     * @param nodeSetEventSource DESCRIBE THE PARAMETER
+     * @param handle DESCRIBE THE PARAMETER
+     * @param added DESCRIBE THE PARAMETER
      */
-    public void update(Observable o, Object arg) {
-      NodeSetUpdate nsu = (NodeSetUpdate) arg;
-
-      NodeHandle handle = nsu.handle();
-      boolean wa = nsu.wasAdded();
-
-      leafSetChange(handle, wa);
+    public void nodeSetUpdate(NodeSetEventSource nodeSetEventSource, NodeHandle handle, boolean added) {
+      leafSetChange(handle, added);
     }
   }
 
@@ -402,20 +419,16 @@ public abstract class PastryAppl implements MessageReceiver {
    * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author jeffh
    */
-  private class RouteSetObserver implements Observer {
+  private class RouteSetObserver implements NodeSetListener {
     /**
      * DESCRIBE THE METHOD
      *
-     * @param o DESCRIBE THE PARAMETER
-     * @param arg DESCRIBE THE PARAMETER
+     * @param nodeSetEventSource DESCRIBE THE PARAMETER
+     * @param handle DESCRIBE THE PARAMETER
+     * @param added DESCRIBE THE PARAMETER
      */
-    public void update(Observable o, Object arg) {
-      NodeSetUpdate nsu = (NodeSetUpdate) arg;
-
-      NodeHandle handle = nsu.handle();
-      boolean wa = nsu.wasAdded();
-
-      routeSetChange(handle, wa);
+    public void nodeSetUpdate(NodeSetEventSource nodeSetEventSource, NodeHandle handle, boolean added) {
+      routeSetChange(handle, added);
     }
   }
 }
