@@ -56,7 +56,7 @@ import rice.selector.TimerTask;
  * This class serves as gluecode, which allows applications written for the
  * common API to work with pastry.
  *
- * @version $Id: PastryEndpoint.java 2843 2005-12-15 12:57:25Z jeffh $
+ * @version $Id: PastryEndpoint.java 3248 2006-04-27 13:04:07Z jeffh $
  * @author Alan Mislove
  * @author Peter Druschel
  */
@@ -85,10 +85,11 @@ public class PastryEndpoint extends PastryAppl implements Endpoint {
    * @param instance DESCRIBE THE PARAMETER
    */
   public PastryEndpoint(PastryNode pn, Application application, String instance) {
-    super(pn, application.getClass().getName() + instance);
+    super(pn, application.getClass().getName() + instance, null);
 
     this.instance = application.getClass().getName() + instance;
     this.application = application;
+    register();
   }
 
   /**
@@ -103,6 +104,7 @@ public class PastryEndpoint extends PastryAppl implements Endpoint {
 
     this.instance = "[PORT " + port + "]";
     this.application = application;
+    register();
   }
 
   // API methods to be invoked by applications
