@@ -48,7 +48,7 @@ import java.io.*;
  * strictly a set: when the ring is small, a node may appear in both the cw and
  * the ccw half of the "set".
  *
- * @version $Id: LeafSet.java 2819 2005-11-28 12:32:42Z jeffh $
+ * @version $Id: LeafSet.java 3246 2006-04-27 13:03:16Z jeffh $
  * @author Andrew Ladd
  * @author Peter Druschel
  */
@@ -204,7 +204,6 @@ public class LeafSet extends Observable implements Serializable {
    * @return The ProperlyRemoved value
    */
   protected boolean isProperlyRemoved(NodeHandle handle) {
-    //if (true) return true;
     return !member(handle);
   }
 
@@ -538,7 +537,7 @@ public class LeafSet extends Observable implements Serializable {
    * @return x mod y
    */
   private int mod(int x, int y) {
-    if (x < 0) {
+    if ((x < 0) ^ (y < 0)) {
       return y + (x % y);
     } else {
       return x % y;
@@ -562,7 +561,7 @@ public class LeafSet extends Observable implements Serializable {
       throw new IllegalArgumentException("Range must be greater than or equal to zero. Attempted " + r);
     }
     if (!(member(n) || baseHandle.equals(n))) {
-      throw new RangeCannotBeDeterminedException("Node " + n + " is not in this leafset.", r, Integer.MIN_VALUE, getUniqueCount(), n, this);
+      throw new LSRangeCannotBeDeterminedException("Node " + n + " is not in this leafset.", r, Integer.MIN_VALUE, getUniqueCount(), n, this);
     }
 
     // get the position of the node and the number of nodes in the network
@@ -603,7 +602,7 @@ public class LeafSet extends Observable implements Serializable {
 
     // if either of it's pair nodes are null, then we cannot determine the range
     if ((ccw == null) || (cw == null)) {
-      throw new RangeCannotBeDeterminedException("This leafset doesn't have enough information to provide the correct range.", r, pos, getUniqueCount(), n, this);
+      throw new LSRangeCannotBeDeterminedException("This leafset doesn't have enough information to provide the correct range.", r, pos, getUniqueCount(), n, this);
     }
 
     // otherwise, we then construct the ranges which comprise the main range, and finally
@@ -636,7 +635,7 @@ public class LeafSet extends Observable implements Serializable {
     IdRange rprev = null;
     try {
       rprev = range(n, r - 1);
-    } catch (RangeCannotBeDeterminedException rcbde) {
+    } catch (LSRangeCannotBeDeterminedException rcbde) {
       // keeps previous functionality now that we are throwing rcbde rather than returning null
     }
     if (rr == null || rprev == null) {
@@ -855,6 +854,7 @@ public class LeafSet extends Observable implements Serializable {
    * Add observer method.
    *
    * @param o the observer to add.
+   * @deprecated use addNodeSetListener
    */
   public void addObserver(Observer o) {
     cwSet.addObserver(o);
@@ -865,12 +865,33 @@ public class LeafSet extends Observable implements Serializable {
    * Delete observer method.
    *
    * @param o the observer to delete.
+   * @deprecated use deleteNodeSetListener
    */
   public void deleteObserver(Observer o) {
     cwSet.deleteObserver(o);
     ccwSet.deleteObserver(o);
   }
 
+  /**
+   * Add observer method.
+   *
+   * @param listener The feature to be added to the NodeSetListener attribute
+   */
+  public void addNodeSetListener(NodeSetListener listener) {
+    cwSet.addNodeSetListener(listener);
+    ccwSet.addNodeSetListener(listener);
+  }
+
+  /**
+   * Delete observer method.
+   *
+   * @param listener DESCRIBE THE PARAMETER
+   */
+  public void deleteNodeSetListener(NodeSetListener listener) {
+    cwSet.removeNodeSetListener(listener);
+    ccwSet.removeNodeSetListener(listener);
+  }
+
   /**
    * Returns a string representation of the leaf set
    *
