@@ -38,6 +38,7 @@ package rice.pastry.routing;
 import rice.environment.Environment;
 import rice.environment.logging.Logger;
 import rice.pastry.*;
+
 import java.util.*;
 
 /**
@@ -64,12 +65,12 @@ import java.util.*;
  * routes for at the <EM>index</EM> th digit and the differing digit is <EM>
  * digit</EM> . An <EM>index</EM> of <EM>0</EM> is the least significant digit.
  *
- * @version $Id: RoutingTable.java 2904 2006-01-10 21:50:37Z jeffh $
+ * @version $Id: RoutingTable.java 2914 2006-01-11 19:53:22Z jeffh $
  * @author Andrew Ladd
  * @author Peter Druschel
  */
 
-public class RoutingTable extends Observable implements Observer {
+public class RoutingTable extends Observable implements NodeSetEventSource {
   /**
    * The routing calculations will occur in base <EM>2 <SUP>idBaseBitLength
    * </SUP></EM>
@@ -91,6 +92,8 @@ public class RoutingTable extends Observable implements Observer {
 
   Logger logger;
 
+  ArrayList listeners = new ArrayList();
+
   /**
    * Constructor.
    *
@@ -117,7 +120,7 @@ public class RoutingTable extends Observable implements Observer {
       // insert this node at the appropriate column
       routingTable[i][myCol] = new RouteSet(maxEntries);
       routingTable[i][myCol].put(myNodeHandle);
-      routingTable[i][myCol].addObserver(this);
+      routingTable[i][myCol].setRoutingTable(this);
     }
   }
 
@@ -346,7 +349,7 @@ public class RoutingTable extends Observable implements Observer {
     if (routingTable[diffDigit][digit] == null) {
       // allocate a RouteSet
       routingTable[diffDigit][digit] = new RouteSet(maxEntries);
-      routingTable[diffDigit][digit].addObserver(this);
+      routingTable[diffDigit][digit].setRoutingTable(this);
     }
 
     return routingTable[diffDigit][digit];
@@ -403,24 +406,27 @@ public class RoutingTable extends Observable implements Observer {
    * changed.
    *
    * @param o the RouteSet
-   * @param arg the event
+   * @param handle DESCRIBE THE PARAMETER
+   * @param added DESCRIBE THE PARAMETER
    */
-  public void update(Observable o, Object arg) {
-    // pass the event to the Observers of this RoutingTable
-    setChanged();
-    notifyObservers(arg);
-  }
+//  public void update(Observable o, Object arg) {
+//    // pass the event to the Observers of this RoutingTable
+//    setChanged();
+//    notifyObservers(arg);
+//  }
 
-  /**
-   * DESCRIBE THE METHOD
-   *
-   * @param o DESCRIBE THE PARAMETER
-   * @param arg DESCRIBE THE PARAMETER
-   */
-  public void update(RouteSet o, Object arg) {
+  public void nodeSetUpdate(Object o, NodeHandle handle, boolean added) {
     // pass the event to the Observers of this RoutingTable
-    setChanged();
-    notifyObservers(arg);
+    synchronized (listeners) {
+      for (int i = 0; i < listeners.size(); i++) {
+        ((NodeSetListener) listeners.get(i)).nodeSetUpdate(this, handle, added);
+      }
+    }
+    // handle deprecated interface
+    if (countObservers() > 0) {
+      setChanged();
+      notifyObservers(new NodeSetUpdate(handle, added));
+    }
   }
 
   /**
@@ -488,4 +494,52 @@ public class RoutingTable extends Observable implements Observer {
     }
     return set.size();
   }
+
+  /**
+   * Generates too many objects to use this interface
+   *
+   * @param o The feature to be added to the Observer attribute
+   * @deprecated use addNodeSetListener
+   */
+  public void addObserver(Observer o) {
+    if (logger.level <= Logger.WARNING) {
+      logger.logException("WARNING: Observer on RoutingTable is deprecated", new Exception("Stack Trace"));
+    }
+    super.addObserver(o);
+  }
+
+  /**
+   * Generates too many objects to use this interface
+   *
+   * @param o DESCRIBE THE PARAMETER
+   * @deprecated use deleteNodeSetListener
+   */
+  public void deleteObserver(Observer o) {
+    if (logger.level <= Logger.WARNING) {
+      logger.log("WARNING: Observer on RoutingTable is deprecated");
+    }
+    super.deleteObserver(o);
+  }
+
+  /**
+   * Adds a feature to the NodeSetListener attribute of the RoutingTable object
+   *
+   * @param listener The feature to be added to the NodeSetListener attribute
+   */
+  public void addNodeSetListener(NodeSetListener listener) {
+    synchronized (listeners) {
+      listeners.add(listener);
+    }
+  }
+
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param listener DESCRIBE THE PARAMETER
+   */
+  public void removeNodeSetListener(NodeSetListener listener) {
+    synchronized (listeners) {
+      listeners.remove(listener);
+    }
+  }
 }
