@@ -41,23 +41,52 @@ import java.nio.*;
 import java.nio.channels.*;
 import java.util.*;
 
+import rice.environment.Environment;
+import rice.environment.logging.Logger;
+import rice.environment.params.Parameters;
+import rice.environment.time.TimeSource;
 import rice.pastry.*;
-import rice.pastry.messaging.*;
 import rice.pastry.socket.messaging.*;
 import rice.selector.*;
 
 /**
- * @version $Id: PingManager.java,v 1.33 2005/05/16 18:30:22 amislove Exp $
+ * @version $Id: PingManager.java 3203 2006-04-17 18:07:26Z jstewart $
  * @author jeffh To change the template for this generated type comment go to
  *      Window>Preferences>Java>Code Generation>Code and Comments
  */
 public class PingManager extends SelectionKeyHandler {
 
-  // InetSocketAddress -> ArrayList of PingResponseListener
+  // whether or not we should use short pings
   /**
    * DESCRIBE THE FIELD
    */
-  protected Hashtable pingListeners = new Hashtable();
+  public final boolean USE_SHORT_PINGS;
+
+  // the size of the buffer used to read incoming datagrams must be big enough
+  // to encompass multiple datagram packets
+  /**
+   * DESCRIBE THE FIELD
+   */
+  public final int DATAGRAM_RECEIVE_BUFFER_SIZE;
+
+  // the size of the buffer used to send outgoing datagrams this is also the
+  // largest message size than can be sent via UDP
+  /**
+   * DESCRIBE THE FIELD
+   */
+  public final int DATAGRAM_SEND_BUFFER_SIZE;
+
+  // SourceRoute -> ArrayList of PingResponseListener
+  /**
+   * DESCRIBE THE FIELD
+   */
+  protected WeakHashMap pingListeners = new WeakHashMap();
+
+  // SourceRoute -> Long
+  /**
+   * DESCRIBE THE FIELD
+   */
+  protected WeakHashMap lastPingTime = new WeakHashMap();
 
   // The list of pending meesages
   /**
@@ -83,14 +112,22 @@ public class PingManager extends SelectionKeyHandler {
   // the local node
   private SocketPastryNode spn;
 
-  // whether or not we should use short pings
+  private Logger logger;
+
+  private TimeSource timeSource;
+
+  private Environment environment;
+
+  long lastTimePrinted = 0;
+
   /**
    * DESCRIBE THE FIELD
    */
-  public static boolean USE_SHORT_PINGS = false;
+  public final static int PING_THROTTLE = 1500;
+  // = false;
 
   // the header which signifies a normal socket
-  /**
+    /**
    * DESCRIBE THE FIELD
    */
   protected static byte[] HEADER_PING = new byte[]{0x49, 0x3A, 0x09, 0x5C};
@@ -113,26 +150,6 @@ public class PingManager extends SelectionKeyHandler {
    */
   public static int HEADER_SIZE = HEADER_PING.length;
 
-  // the size of the buffer used to read incoming datagrams must be big enough
-  // to encompass multiple datagram packets
-  /**
-   * DESCRIBE THE FIELD
-   */
-  public static int DATAGRAM_RECEIVE_BUFFER_SIZE = 131072;
-
-  // the size of the buffer used to send outgoing datagrams this is also the
-  // largest message size than can be sent via UDP
-  /**
-   * DESCRIBE THE FIELD
-   */
-  public static int DATAGRAM_SEND_BUFFER_SIZE = 65536;
-
-  // the ping throttle, or how often to actually ping a remote node
-  /**
-   * DESCRIBE THE FIELD
-   */
-  public static int PING_THROTTLE = 600000;
-
   /**
    * @param manager DESCRIBE THE PARAMETER
    * @param spn DESCRIBE THE PARAMETER
@@ -141,9 +158,17 @@ public class PingManager extends SelectionKeyHandler {
    */
   public PingManager(SocketPastryNode spn, SocketSourceRouteManager manager, EpochInetSocketAddress bindAddress, EpochInetSocketAddress proxyAddress) {
     this.spn = spn;
+    this.environment = spn.getEnvironment();
+    this.logger = environment.getLogManager().getLogger(PingManager.class, null);
+    this.timeSource = environment.getTimeSource();
+
+    Parameters p = environment.getParameters();
     this.manager = manager;
     this.pendingMsgs = new ArrayList();
     this.localAddress = proxyAddress;
+    USE_SHORT_PINGS = p.getBoolean("pastry_socket_pingmanager_smallPings");
+    DATAGRAM_RECEIVE_BUFFER_SIZE = p.getInt("pastry_socket_pingmanager_datagram_receive_buffer_size");
+    DATAGRAM_SEND_BUFFER_SIZE = p.getInt("pastry_socket_pingmanager_datagram_send_buffer_size");
 
     // allocate enought bytes to read data
     this.buffer = ByteBuffer.allocateDirect(DATAGRAM_SEND_BUFFER_SIZE);
@@ -152,14 +177,18 @@ public class PingManager extends SelectionKeyHandler {
       // bind to the appropriate port
       channel = DatagramChannel.open();
       channel.configureBlocking(false);
+      channel.socket().setReuseAddress(true);
       channel.socket().bind(bindAddress.getAddress());
       channel.socket().setSendBufferSize(DATAGRAM_SEND_BUFFER_SIZE);
       channel.socket().setReceiveBufferSize(DATAGRAM_RECEIVE_BUFFER_SIZE);
 
-      key = SelectorManager.getSelectorManager().register(channel, this, 0);
+      key = environment.getSelectorManager().register(channel, this, 0);
       key.interestOps(SelectionKey.OP_READ);
     } catch (IOException e) {
-      System.out.println("PANIC: Error binding datagram server to address " + localAddress + ": " + e);
+      if (logger.level <= Logger.SEVERE) {
+        logger.log(
+          "PANIC: Error binding datagram server to address " + localAddress + ": " + e);
+      }
     }
   }
 
@@ -179,14 +208,37 @@ public class PingManager extends SelectionKeyHandler {
    * @param prl The listener which should hear about the response
    */
   protected void ping(SourceRoute path, PingResponseListener prl) {
-    debug("Actually sending ping via path " + path + " local " + localAddress);
+    // this code is to throttle pings
+    // I don't know what to do if there is a prl, because it is difficult to know
+    // if there is still an outstanding ping, so we can only throttle if there is no
+    // prl
+    long curTime = timeSource.currentTimeMillis();
+    if (prl == null) {
+      Long time = (Long) lastPingTime.get(path);
+      if (time != null) {
+        if ((time.longValue() + PING_THROTTLE) > curTime) {
+          if (logger.level <= Logger.FINE) {
+            logger.log(
+              "(PM) Suppressing ping via path " + path + " local " + localAddress);
+          }
+          return;
+        }
+      }
+    }
+
+    if (logger.level <= Logger.FINE) {
+      logger.log(
+        "(PM) Actually sending ping via path " + path + " local " + localAddress);
+    }
+
+    lastPingTime.put(path, new Long(curTime));
 
     addPingResponseListener(path, prl);
 
     if (USE_SHORT_PINGS) {
       sendShortPing(path);
     } else {
-      enqueue(path, new PingMessage(path, path.reverse(localAddress)));
+      enqueue(path, new PingMessage(path, path.reverse(localAddress), environment.getTimeSource().currentTimeMillis()));
     }
   }
 
@@ -197,8 +249,12 @@ public class PingManager extends SelectionKeyHandler {
    * @exception IOException DESCRIBE THE EXCEPTION
    */
   protected void resign() throws IOException {
-    key.channel().close();
-    key.cancel();
+    if (key != null) {
+      if (key.channel() != null) {
+        key.channel().close();
+      }
+      key.cancel();
+    }
   }
 
   /**
@@ -225,13 +281,16 @@ public class PingManager extends SelectionKeyHandler {
       DataOutputStream dos = new DataOutputStream(baos);
 
       dos.write(HEADER_SHORT_PING);
-      dos.writeLong(System.currentTimeMillis());
+      dos.writeLong(environment.getTimeSource().currentTimeMillis());
 
       dos.flush();
 
       enqueue(route, baos.toByteArray());
     } catch (Exception canthappen) {
-      System.out.println("CANT HAPPEN: " + canthappen);
+      if (logger.level <= Logger.SEVERE) {
+        logger.log(
+          "CANT HAPPEN: " + canthappen);
+      }
     }
   }
 
@@ -258,7 +317,7 @@ public class PingManager extends SelectionKeyHandler {
     DataInputStream dis = new DataInputStream(new ByteArrayInputStream(payload));
     dis.readFully(new byte[HEADER_SHORT_PING_RESPONSE.length]);
     long start = dis.readLong();
-    int ping = (int) (System.currentTimeMillis() - start);
+    int ping = (int) (environment.getTimeSource().currentTimeMillis() - start);
 
     SourceRoute from = route.reverse();
 
@@ -267,6 +326,28 @@ public class PingManager extends SelectionKeyHandler {
     notifyPingResponseListeners(from, ping, start);
   }
 
+  /**
+   * Adds a feature to the PingResponseListener attribute of the PingManager
+   * object
+   *
+   * @param prl The feature to be added to the PingResponseListener attribute
+   * @param path DESCRIBE THE PARAMETER
+   */
+  protected void removePingResponseListener(SourceRoute path, PingResponseListener prl) {
+    if (prl == null) {
+      return;
+    }
+
+    ArrayList list = (ArrayList) pingListeners.get(path);
+
+    if (list != null) {
+      // remove all
+      while (list.remove(prl)) {
+        ;
+      }
+    }
+  }
+
   /**
    * Adds a feature to the PingResponseListener attribute of the PingManager
    * object
@@ -316,29 +397,39 @@ public class PingManager extends SelectionKeyHandler {
    */
   public void enqueue(SourceRoute path, Object msg) {
     try {
-      byte[] data = addHeader(path, msg, localAddress);
+      byte[] data = addHeader(path, msg, localAddress, environment, logger);
 
       synchronized (pendingMsgs) {
         pendingMsgs.add(new Envelope(path.getFirstHop(), data));
       }
 
-      if ((spn != null) && (spn instanceof SocketPastryNode)) {
-        ((SocketPastryNode) spn).broadcastSentListeners(msg, path.toArray(), data.length);
+      if (spn != null) {
+        ((SocketPastryNode) spn).broadcastSentListeners(msg, path.getLastHop().address, data.length, NetworkListener.TYPE_UDP);
       }
 
-      if (SocketPastryNode.verbose) {
-        if (!(msg instanceof byte[])) {
-          System.out.println("COUNT: " + System.currentTimeMillis() + " Sent message " + msg.getClass() + " of size " + data.length + " to " + path);
-        } else if (((byte[]) msg)[3] == 0x11) {
-          System.out.println("COUNT: " + System.currentTimeMillis() + " Sent message rice.pastry.socket.messaging.ShortPingMessage of size " + data.length + " to " + path);
-        } else if (((byte[]) msg)[3] == 0x12) {
-          System.out.println("COUNT: " + System.currentTimeMillis() + " Sent message rice.pastry.socket.messaging.ShortPingResponseMessage of size " + data.length + " to " + path);
+      if (!(msg instanceof byte[])) {
+        if (logger.level <= Logger.FINER) {
+          logger.log(
+            "COUNT: Sent message " + msg.getClass() + " of size " + data.length + " to " + path);
+        }
+      } else if (((byte[]) msg)[3] == 0x11) {
+        if (logger.level <= Logger.FINER) {
+          logger.log(
+            "COUNT: Sent message rice.pastry.socket.messaging.ShortPingMessage of size " + data.length + " to " + path);
+        }
+      } else if (((byte[]) msg)[3] == 0x12) {
+        if (logger.level <= Logger.FINER) {
+          logger.log(
+            "COUNT: Sent message rice.pastry.socket.messaging.ShortPingResponseMessage of size " + data.length + " to " + path);
         }
       }
 
-      SelectorManager.getSelectorManager().modifyKey(key);
+      environment.getSelectorManager().modifyKey(key);
     } catch (IOException e) {
-      System.out.println("ERROR: Received exceptoin " + e + " while enqueuing ping " + msg);
+      if (logger.level <= Logger.SEVERE) {
+        logger.log(
+          "ERROR: Received exceptoin " + e + " while enqueuing ping " + msg);
+      }
     }
   }
 
@@ -360,21 +451,23 @@ public class PingManager extends SelectionKeyHandler {
         path = SourceRoute.build(new EpochInetSocketAddress(from));
       }
 
-      if ((spn != null) && (spn instanceof SocketPastryNode)) {
-        ((SocketPastryNode) spn).broadcastReceivedListeners(dm, path.reverse().toArray(), size);
+      if (spn != null) {
+        ((SocketPastryNode) spn).broadcastReceivedListeners(dm, path.reverse().getLastHop().address, size, NetworkListener.TYPE_UDP);
       }
 
       if (dm instanceof PingMessage) {
-        if (SocketPastryNode.verbose) {
-          System.out.println("COUNT: " + System.currentTimeMillis() + " Read message " + message.getClass() + " of size " + size + " from " + dm.getInboundPath().reverse());
+        if (logger.level <= Logger.FINER) {
+          logger.log(
+            "COUNT: Read message " + message.getClass() + " of size " + size + " from " + dm.getInboundPath().reverse());
         }
 
         enqueue(dm.getInboundPath(), new PingResponseMessage(dm.getOutboundPath(), dm.getInboundPath(), start));
       } else if (dm instanceof PingResponseMessage) {
-        if (SocketPastryNode.verbose) {
-          System.out.println("COUNT: " + System.currentTimeMillis() + " Read message " + message.getClass() + " of size " + size + " from " + dm.getOutboundPath().reverse());
+        if (logger.level <= Logger.FINER) {
+          logger.log(
+            "COUNT: Read message " + message.getClass() + " of size " + size + " from " + dm.getOutboundPath().reverse());
         }
-        int ping = (int) (System.currentTimeMillis() - start);
+        int ping = (int) (environment.getTimeSource().currentTimeMillis() - start);
 
         manager.markAlive(dm.getOutboundPath());
         manager.markProximity(dm.getOutboundPath(), ping);
@@ -382,20 +475,25 @@ public class PingManager extends SelectionKeyHandler {
       } else if (dm instanceof WrongEpochMessage) {
         WrongEpochMessage wem = (WrongEpochMessage) dm;
 
-        if (SocketPastryNode.verbose) {
-          System.out.println("COUNT: " + System.currentTimeMillis() + " Read message " + message.getClass() + " of size " + size + " from " + dm.getOutboundPath().reverse());
+        if (logger.level <= Logger.FINER) {
+          logger.log(
+            "COUNT: Read message " + message.getClass() + " of size " + size + " from " + dm.getOutboundPath().reverse());
         }
 
         manager.markAlive(dm.getOutboundPath());
         manager.markDead(wem.getIncorrect());
       } else if (dm instanceof IPAddressRequestMessage) {
-        if (SocketPastryNode.verbose) {
-          System.out.println("COUNT: " + System.currentTimeMillis() + " Read message " + message.getClass() + " of size " + size + " from " + SourceRoute.build(new EpochInetSocketAddress(from)));
+        if (logger.level <= Logger.FINER) {
+          logger.log(
+            "COUNT: Read message " + message.getClass() + " of size " + size + " from " + SourceRoute.build(new EpochInetSocketAddress(from)));
         }
 
-        enqueue(SourceRoute.build(new EpochInetSocketAddress(from)), new IPAddressResponseMessage(from));
+        enqueue(SourceRoute.build(new EpochInetSocketAddress(from)), new IPAddressResponseMessage(from, environment.getTimeSource().currentTimeMillis()));
       } else {
-        System.out.println("ERROR: Received unknown DatagramMessage " + dm);
+        if (logger.level <= Logger.WARNING) {
+          logger.log(
+            "ERROR: Received unknown DatagramMessage " + dm);
+        }
       }
     }
   }
@@ -415,20 +513,25 @@ public class PingManager extends SelectionKeyHandler {
         /*
          *  if (address.getPort() % 2 == localAddress.getAddress().getPort() % 2) {
          *  buffer.clear();
-         *  System.out.println("Dropping packet");
+         *  log(Logger.INFO,"Dropping packet");
          *  return;
          *  }
          */
         if (buffer.remaining() > 0) {
           readHeader(address);
         } else {
-          debug("Read from datagram channel, but no bytes were there - no bad, but wierd.");
+          if (logger.level <= Logger.INFO) {
+            logger.log(
+              "(PM) Read from datagram channel, but no bytes were there - no bad, but wierd.");
+          }
           break;
         }
       }
     } catch (IOException e) {
-      System.out.println("ERROR (datagrammanager:read): " + e);
-      e.printStackTrace();
+      if (logger.level <= Logger.WARNING) {
+        logger.logException(
+          "ERROR (datagrammanager:read): ", e);
+      }
     } finally {
       buffer.clear();
     }
@@ -440,22 +543,40 @@ public class PingManager extends SelectionKeyHandler {
    * @param key DESCRIBE THE PARAMETER
    */
   public void write(SelectionKey key) {
+    Envelope write = null;
+
     try {
       synchronized (pendingMsgs) {
         Iterator i = pendingMsgs.iterator();
 
         while (i.hasNext()) {
-          Envelope write = (Envelope) i.next();
+          write = (Envelope) i.next();
 
-          if (channel.send(ByteBuffer.wrap(write.data), write.destination.getAddress()) == write.data.length) {
+          try {
+            if (channel.send(ByteBuffer.wrap(write.data), write.destination.getAddress()) == write.data.length) {
+              i.remove();
+            } else {
+              break;
+            }
+          } catch (IOException e) {
             i.remove();
-          } else {
-            break;
+            throw e;
           }
         }
       }
     } catch (IOException e) {
-      System.err.println("ERROR (datagrammanager:write): " + e);
+      if (logger.level <= Logger.WARNING) {
+        // This code prevents this line from filling up logs during some kinds of network outages
+        // it makes this error only be printed 1ce/second
+        long now = timeSource.currentTimeMillis();
+        if (lastTimePrinted + 1000 > now) {
+          return;
+        }
+        lastTimePrinted = now;
+
+        logger.logException(
+          "ERROR (datagrammanager:write) to " + (write == null ? null : write.destination.getAddress()), e);
+      }
     } finally {
       if (pendingMsgs.isEmpty()) {
         key.interestOps(key.interestOps() & ~SelectionKey.OP_WRITE);
@@ -476,6 +597,23 @@ public class PingManager extends SelectionKeyHandler {
     }
   }
 
+  /**
+   * Method which adds a header for the provided path to the given data.
+   *
+   * @param header DESCRIBE THE PARAMETER
+   * @return The messag with a header attached
+   * @exception IOException DESCRIBE THE EXCEPTION
+   */
+  public SourceRoute decodeHeader(byte[] header) throws IOException {
+    EpochInetSocketAddress[] route = new EpochInetSocketAddress[header.length / SocketChannelRepeater.HEADER_BUFFER_SIZE];
+
+    for (int i = 0; i < route.length; i++) {
+      route[i] = SocketChannelRepeater.decodeHeader(header, i);
+    }
+
+    return SourceRoute.build(route);
+  }
+
   /**
    * Method which processes an incoming message and hands it off to the
    * appropriate handler.
@@ -504,6 +642,7 @@ public class PingManager extends SelectionKeyHandler {
         // if the packet is at the end of the route, accept it
         // otherwise, forward it to the next hop (and increment the stamp)
         if (metadata[0] + 1 == metadata[1]) {
+          // The message was meant for me
           byte[] array = new byte[buffer.remaining()];
           buffer.get(array);
           buffer.clear();
@@ -514,19 +653,34 @@ public class PingManager extends SelectionKeyHandler {
           SourceRoute sr = decodeHeader(route).removeLastHop();
 
           if (Arrays.equals(test, HEADER_SHORT_PING)) {
-            if (SocketPastryNode.verbose) {
-              System.out.println("COUNT: " + System.currentTimeMillis() + " Read message rice.pastry.socket.messaging.ShortPingMessage of size " + (header.length + metadata.length + array.length + route.length) + " from " + sr);
+            // the PING was meant for me
+            int len = (header.length + metadata.length + array.length + route.length);
+            if (logger.level <= Logger.FINER) {
+              logger.log(
+                "COUNT: Read message rice.pastry.socket.messaging.ShortPingMessage of size " + len + " from " + sr);
             }
+            if (spn != null) {
+              ((SocketPastryNode) spn).broadcastReceivedListeners(array, address, len, NetworkListener.TYPE_UDP);
+            }
+
             shortPingReceived(sr, array);
           } else if (Arrays.equals(test, HEADER_SHORT_PING_RESPONSE)) {
-            if (SocketPastryNode.verbose) {
-              System.out.println("COUNT: " + System.currentTimeMillis() + " Read message rice.pastry.socket.messaging.ShortPingResponseMessage of size " + (header.length + metadata.length + array.length + route.length) + " from " + sr);
+            // the PING_RESPONSE was meant for me
+            int len = (header.length + metadata.length + array.length + route.length);
+            if (logger.level <= Logger.FINER) {
+              logger.log(
+                "COUNT: Read message rice.pastry.socket.messaging.ShortPingResponseMessage of size " + len + " from " + sr);
+            }
+
+            if (spn != null) {
+              ((SocketPastryNode) spn).broadcastReceivedListeners(array, address, len, NetworkListener.TYPE_UDP);
             }
             shortPingResponseReceived(sr, array);
           } else {
-            receiveMessage(deserialize(array), array.length, address);
+            receiveMessage(deserialize(array, environment, spn, logger), array.length, address);
           }
         } else {
+          // sourceroute hop
           EpochInetSocketAddress next = SocketChannelRepeater.decodeHeader(route, metadata[0] + 1);
           buffer.position(0);
           byte[] packet = new byte[buffer.remaining()];
@@ -535,11 +689,16 @@ public class PingManager extends SelectionKeyHandler {
           // increment the hop count
           packet[HEADER_SIZE]++;
 
+          if (spn != null) {
+            ((SocketPastryNode) spn).broadcastReceivedListeners(packet, address, packet.length, NetworkListener.TYPE_SR_UDP);
+            ((SocketPastryNode) spn).broadcastSentListeners(packet, next.address, packet.length, NetworkListener.TYPE_SR_UDP);
+          }
+
           synchronized (pendingMsgs) {
             pendingMsgs.add(new Envelope(next, packet));
           }
 
-          SelectorManager.getSelectorManager().modifyKey(key);
+          environment.getSelectorManager().modifyKey(key);
         }
       } else {
         // if this is an old epoch of ours, reply with an update
@@ -555,51 +714,59 @@ public class PingManager extends SelectionKeyHandler {
           }
 
           outbound = outbound.append(localAddress);
+//          if (spn != null) {
+//            ((SocketPastryNode) spn).broadcastReceivedListeners(packet, address, packet.length, NetworkListener.TYPE_SR_UDP);
+//          }
 
-          enqueue(back.reverse(), new WrongEpochMessage(outbound, back.reverse(), eisa, localAddress));
+          enqueue(back.reverse(), new WrongEpochMessage(outbound, back.reverse(), eisa, localAddress, environment.getTimeSource().currentTimeMillis()));
         } else {
-          System.out.println("WARNING: Received packet destined for EISA (" + metadata[0] + " " + metadata[1] + ") " + eisa + " but the local address is " + localAddress + " - dropping silently.");
+          if (logger.level <= Logger.WARNING) {
+            logger.log(
+              "WARNING: Received packet destined for EISA (" + metadata[0] + " " + metadata[1] + ") " + eisa + " but the local address is " + localAddress + " - dropping silently.");
+          }
           throw new IOException("Received packet destined for EISA (" + metadata[0] + " " + metadata[1] + ") " + eisa + " but the local address is " + localAddress + " - dropping silently.");
         }
       }
     } else {
-      System.out.println("WARNING: Received unrecognized message header - ignoring from " + address + ".");
+      if (logger.level <= Logger.WARNING) {
+        logger.log(
+          "WARNING: Received unrecognized message header - ignoring from " + address + ".");
+      }
       throw new IOException("Improper message header received - ignoring from " + address + ". Read " + ((byte) header[0]) + " " + ((byte) header[1]) + " " + ((byte) header[2]) + " " + ((byte) header[3]));
     }
   }
 
-  /**
-   * Debug method
-   *
-   * @param s The string to print
-   */
-  private void debug(String s) {
-    if (Log.ifp(8)) {
-      System.out.println(spn.getNodeId() + " (PM): " + s);
-    }
-  }
-
   /**
    * Method which serializes a given object into a ByteBuffer, in order to
    * prepare it for writing.
    *
    * @param message DESCRIBE THE PARAMETER
+   * @param environment DESCRIBE THE PARAMETER
+   * @param logger DESCRIBE THE PARAMETER
    * @return A ByteBuffer containing the object
    * @exception IOException if the object can't be serialized
    */
-  public static byte[] serialize(Object message) throws IOException {
+  public static byte[] serialize(Object message, Environment environment, Logger logger) throws IOException {
     try {
+
       ByteArrayOutputStream baos = new ByteArrayOutputStream();
       ObjectOutputStream oos = new ObjectOutputStream(baos);
       oos.writeObject(message);
       oos.close();
 
-      return baos.toByteArray();
+      byte[] ret = baos.toByteArray();
+      return ret;
     } catch (InvalidClassException e) {
-      System.out.println("PANIC: Object to be serialized was an invalid class!");
+      if (logger.level <= Logger.SEVERE) {
+        logger.logException(
+          "PANIC: Object to be serialized was an invalid class!", e);
+      }
       throw new IOException("Invalid class during attempt to serialize.");
     } catch (NotSerializableException e) {
-      System.out.println("PANIC: Object to be serialized was not serializable! [" + message + "]");
+      if (logger.level <= Logger.SEVERE) {
+        logger.logException(
+          "PANIC: Object to be serialized was not serializable! [" + message + "]", e);
+      }
       throw new IOException("Unserializable class " + message + " during attempt to serialize.");
     }
   }
@@ -609,19 +776,29 @@ public class PingManager extends SelectionKeyHandler {
    * the contained object.
    *
    * @param array DESCRIBE THE PARAMETER
+   * @param env DESCRIBE THE PARAMETER
+   * @param spn DESCRIBE THE PARAMETER
+   * @param logger DESCRIBE THE PARAMETER
    * @return The deserialized object.
    * @exception IOException if the buffer can't be deserialized
    */
-  public static Object deserialize(byte[] array) throws IOException {
-    ObjectInputStream ois = new ObjectInputStream(new ByteArrayInputStream(array));
+  public static Object deserialize(byte[] array, Environment env, SocketPastryNode spn, Logger logger) throws IOException {
+    PastryObjectInputStream ois = new PastryObjectInputStream(new ByteArrayInputStream(array), spn);
 
     try {
-      return ois.readObject();
+      Object ret = ois.readObject();
+      return ret;
     } catch (ClassNotFoundException e) {
-      System.out.println("PANIC: Unknown class type in serialized message!");
+      if (logger.level <= Logger.SEVERE) {
+        logger.logException(
+          "PANIC: Unknown class type in serialized message!", e);
+      }
       throw new IOException("Unknown class type in message - closing channel.");
     } catch (InvalidClassException e) {
-      System.out.println("PANIC: Serialized message was an invalid class!");
+      if (logger.level <= Logger.SEVERE) {
+        logger.logException(
+          "PANIC: Serialized message was an invalid class!", e);
+      }
       throw new IOException("Invalid class in message - closing channel.");
     }
   }
@@ -632,10 +809,12 @@ public class PingManager extends SelectionKeyHandler {
    * @param path The feature to be added to the Header attribute
    * @param data The feature to be added to the Header attribute
    * @param localAddress The feature to be added to the Header attribute
+   * @param env The feature to be added to the Header attribute
+   * @param logger The feature to be added to the Header attribute
    * @return The messag with a header attached
    * @exception IOException DESCRIBE THE EXCEPTION
    */
-  public static byte[] addHeader(SourceRoute path, Object data, EpochInetSocketAddress localAddress) throws IOException {
+  public static byte[] addHeader(SourceRoute path, Object data, EpochInetSocketAddress localAddress, Environment env, Logger logger) throws IOException {
     ByteArrayOutputStream baos = new ByteArrayOutputStream();
     DataOutputStream dos = new DataOutputStream(baos);
 
@@ -651,7 +830,7 @@ public class PingManager extends SelectionKeyHandler {
     if (data instanceof byte[]) {
       dos.write((byte[]) data);
     } else {
-      dos.write(serialize(data));
+      dos.write(serialize(data, env, logger));
     }
 
     dos.flush();
@@ -659,27 +838,10 @@ public class PingManager extends SelectionKeyHandler {
     return baos.toByteArray();
   }
 
-  /**
-   * Method which adds a header for the provided path to the given data.
-   *
-   * @param header DESCRIBE THE PARAMETER
-   * @return The messag with a header attached
-   * @exception IOException DESCRIBE THE EXCEPTION
-   */
-  public static SourceRoute decodeHeader(byte[] header) throws IOException {
-    EpochInetSocketAddress[] route = new EpochInetSocketAddress[header.length / SocketChannelRepeater.HEADER_BUFFER_SIZE];
-
-    for (int i = 0; i < route.length; i++) {
-      route[i] = SocketChannelRepeater.decodeHeader(header, i);
-    }
-
-    return SourceRoute.build(route);
-  }
-
   /**
    * Internal class which holds a pending datagram
    *
-   * @version $Id: pretty.settings,v 1.3 2005/03/11 20:22:33 jeffh Exp $
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
    * @author amislove
    */
   public class Envelope {
@@ -703,5 +865,4 @@ public class PingManager extends SelectionKeyHandler {
       this.data = data;
     }
   }
-
 }
