@@ -176,7 +176,7 @@ public class SocketChannelWriter {
       } else if (queue.size() > 20) {
         if (logger.level <= Logger.WARNING) {
           logger.log(
-            "ERROR: Queue to " + path + " has " + queue.size() + " elements (more than 20) - probably a bad sign - enqueue of " + o);
+            "ERROR: Queue to " + path + " has " + queue.size() + " elements (more than " + MAXIMUM_QUEUE_LENGTH + ") - probably a bad sign - enqueue of " + o);
         }
       }
     }
@@ -255,9 +255,7 @@ public class SocketChannelWriter {
               }
               record("Sent", queue.getFirst(), buffer.limit(), path);
             } else {
-              synchronized (queue) {
-                queue.removeFirst();
-              }
+              queue.removeFirst();
 
               return write(sc);
             }
@@ -285,10 +283,8 @@ public class SocketChannelWriter {
             "(W) Finished writing message " + queue.getFirst() + " - queue now contains " + (queue.size() - 1) + " items");
         }
 
-        synchronized (queue) {
-          queue.removeFirst();
-          buffer = null;
-        }
+        queue.removeFirst();
+        buffer = null;
       }
     }
   }
@@ -364,25 +360,25 @@ public class SocketChannelWriter {
     } catch (InvalidClassException e) {
       if (logger.level <= Logger.SEVERE) {
         logger.log(
-          "PANIC: Object to be serialized was an invalid class!");
+          "PANIC: Object to be serialized was an invalid class! [" + o + "]" + e);
       }
       throw new IOException("Invalid class during attempt to serialize.");
     } catch (NotSerializableException e) {
       if (logger.level <= Logger.SEVERE) {
         logger.log(
-          "PANIC: Object to be serialized was not serializable! [" + o + "]");
+          "PANIC: Object to be serialized was not serializable! [" + o + "]" + e);
       }
       throw new IOException("Unserializable class during attempt to serialize.");
     } catch (NullPointerException e) {
       if (logger.level <= Logger.SEVERE) {
-        logger.logException(
-          "PANIC: Object to be serialized caused null pointer exception! [" + o + "]", e);
+        logger.log(
+          "PANIC: Object to be serialized caused null pointer exception! [" + o + "]" + e);
       }
       return null;
     } catch (Exception e) {
       if (logger.level <= Logger.SEVERE) {
         logger.log(
-          "PANIC: Object to be serialized caused excception! [" + e + "]");
+          "PANIC: Object to be serialized caused excception! [" + e + "]" + e);
       }
       throw new IOException("Exception during attempt to serialize.");
     }
