@@ -89,11 +89,11 @@ public class SocketCollectionManager extends SelectionKeyHandler {
    */
   public final int PING_DELAY;
 
-  // how much jitter to add to the ping waits - we may wait up to this time before giving up
+  // factor of jitter to adjust to the ping waits - we may wait up to this time before giving up
   /**
    * DESCRIBE THE FIELD
    */
-  public final int PING_JITTER;
+  public final float PING_JITTER;
 
   // how many tries to ping before giving up
   /**
@@ -214,7 +214,7 @@ public class SocketCollectionManager extends SelectionKeyHandler {
     MAX_OPEN_SOURCE_ROUTES = p.getInt("pastry_socket_scm_max_open_source_routes");
     SOCKET_BUFFER_SIZE = p.getInt("pastry_socket_scm_socket_buffer_size");
     PING_DELAY = p.getInt("pastry_socket_scm_ping_delay");
-    PING_JITTER = p.getInt("pastry_socket_scm_ping_jitter");
+    PING_JITTER = p.getFloat("pastry_socket_scm_ping_jitter");
     NUM_PING_TRIES = p.getInt("pastry_socket_scm_num_ping_tries");
     WRITE_WAIT_TIME = p.getInt("pastry_socket_scm_write_wait_time");
     BACKOFF_INITIAL = p.getInt("pastry_socket_scm_backoff_initial");
@@ -228,6 +228,7 @@ public class SocketCollectionManager extends SelectionKeyHandler {
       // bind to port
       final ServerSocketChannel channel = ServerSocketChannel.open();
       channel.configureBlocking(false);
+      channel.socket().setReuseAddress(true);
       channel.socket().bind(bindAddress.getAddress());
 
       this.key = pastryNode.getEnvironment().getSelectorManager().register(channel, this, 0);
@@ -352,7 +353,13 @@ public class SocketCollectionManager extends SelectionKeyHandler {
         logger.log("CHECK DEAD: " + localAddress + " CHECKING DEATH OF PATH " + path);
       }
       DeadChecker checker = new DeadChecker(path, NUM_PING_TRIES);
-      ((SocketPastryNode) pastryNode).getTimer().scheduleAtFixedRate(checker, PING_DELAY + random.nextInt(PING_JITTER), PING_DELAY + random.nextInt(PING_JITTER));
+//      ((SocketPastryNode) pastryNode).getTimer().scheduleAtFixedRate(checker, PING_DELAY + random.nextInt(PING_JITTER), PING_DELAY + random.nextInt(PING_JITTER));
+
+      int delay = manager.proximity(path) * 2;
+      if (delay > PING_DELAY) {
+        delay = PING_DELAY;
+      }
+      ((SocketPastryNode) pastryNode).getTimer().schedule(checker, delay);
       pingManager.ping(path, checker);
     }
   }
@@ -801,7 +808,7 @@ public class SocketCollectionManager extends SelectionKeyHandler {
   /**
    * DESCRIBE THE CLASS
    *
-   * @version $Id: SocketCollectionManager.java 2870 2005-12-21 15:07:14Z jeffh
+   * @version $Id: SocketCollectionManager.java 3251 2006-04-27 13:11:05Z jeffh
    *      $
    * @author jeffh
    */
@@ -856,7 +863,11 @@ public class SocketCollectionManager extends SelectionKeyHandler {
     }
 
     /**
-     * Main processing method for the DeadChecker object
+     * Main processing method for the DeadChecker object value of tries before
+     * run() is called:the time since ping was called:the time since deadchecker
+     * was started 1:500:500 2:1000:1500 3:2000:3500 4:4000:7500 5:8000:15500 //
+     * ~15 seconds to find 1 path faulty, using source routes gives us 30
+     * seconds to find a node faulty
      */
     public void run() {
       if (tries < numTries) {
@@ -866,6 +877,10 @@ public class SocketCollectionManager extends SelectionKeyHandler {
         }
 
         pingManager.ping(path, this);
+        int absPD = (int) (PING_DELAY * Math.pow(2, tries - 1));
+        int jitterAmt = (int) (((float) absPD) * PING_JITTER);
+        int scheduledTime = absPD - jitterAmt + random.nextInt(jitterAmt * 2);
+        ((SocketPastryNode) pastryNode).getTimer().schedule(this, scheduledTime);
       } else {
         if (logger.level <= Logger.FINE) {
           logger.log("DeadChecker(" + path + ") expired - marking as dead.");
@@ -893,7 +908,7 @@ public class SocketCollectionManager extends SelectionKeyHandler {
    * from, and allows the connected to hand the socket off the appropriate node
    * handle.
    *
-   * @version $Id: SocketCollectionManager.java 2870 2005-12-21 15:07:14Z jeffh
+   * @version $Id: SocketCollectionManager.java 3251 2006-04-27 13:11:05Z jeffh
    *      $
    * @author jeffh
    */
@@ -1341,7 +1356,7 @@ public class SocketCollectionManager extends SelectionKeyHandler {
    * between them. It also is responsible for performing the initial handshake
    * and sending the data across the wire.
    *
-   * @version $Id: SocketCollectionManager.java 2870 2005-12-21 15:07:14Z jeffh
+   * @version $Id: SocketCollectionManager.java 3251 2006-04-27 13:11:05Z jeffh
    *      $
    * @author jeffh
    */
