@@ -36,16 +36,14 @@ if advised of the possibility of such damage.
 
 package rice.pastry.socket;
 
-import java.io.*;
-import java.net.*;
+import java.net.InetSocketAddress;
 import java.util.*;
 
 import rice.environment.logging.Logger;
-import rice.pastry.*;
-import rice.pastry.dist.*;
-import rice.pastry.messaging.*;
+import rice.pastry.NodeId;
+import rice.pastry.dist.DistNodeHandle;
+import rice.pastry.messaging.Message;
 import rice.pastry.socket.SocketSourceRouteManager.AddressManager;
-import rice.selector.SelectorManager;
 
 /**
  * Class which represents the address and nodeId of a remote node. In the socket
@@ -53,7 +51,7 @@ import rice.selector.SelectorManager;
  * managed by the local nodes. SocketNodeHandle can now internally exist without
  * a NodeId. This is to get the memory management correct
  *
- * @version $Id: SocketNodeHandle.java 2826 2005-11-30 16:39:45Z jeffh $
+ * @version $Id: SocketNodeHandle.java 3197 2006-04-11 12:30:58Z jeffh $
  * @author Alan Mislove
  */
 public class SocketNodeHandle extends DistNodeHandle {
@@ -110,12 +108,20 @@ public class SocketNodeHandle extends DistNodeHandle {
   public int getLiveness() {
     SocketPastryNode spn = (SocketPastryNode) getLocalNode();
 
-    if (spn == null) {
+    if (spn == null || getLocalNode().getLocalHandle() == null) {
       return LIVENESS_ALIVE;
     } else {
       if (isLocal()) {
         return LIVENESS_ALIVE;
       } else {
+        // make sure this isn't an old existance of ourself:
+        EpochInetSocketAddress localEaddr = ((SocketNodeHandle) getLocalNode().getLocalHandle()).getEpochAddress();
+        InetSocketAddress localAddress = localEaddr.getAddress();
+        InetSocketAddress myAddress = getEpochAddress().getAddress();
+        if (localAddress.equals(myAddress) && (!getEpochAddress().equals(localEaddr))) {
+          return LIVENESS_DEAD_FOREVER;
+        }
+
         return spn.getSocketSourceRouteManager().getLiveness(getEpochAddress());
       }
     }
@@ -128,7 +134,7 @@ public class SocketNodeHandle extends DistNodeHandle {
    */
   public boolean isLocal() {
     assertLocalNode();
-    return getLocalNode().getNodeId().equals(nodeId);
+    return getLocalNode().getLocalHandle().equals(this);
   }
 
   /**
@@ -313,6 +319,24 @@ public class SocketNodeHandle extends DistNodeHandle {
   public void update(Observable o, Object obj) {
   }
 
+//  transient HashSet obs = new HashSet();
+//  public void addObserver(Observer o) {
+//    if (obs == null) obs = new HashSet();
+//    if (logger.level <= Logger.FINER) logger.log(this+".addObserver("+o+")");
+//    synchronized (obs) {
+//      obs.add(o);
+//    }
+//    super.addObserver(o);
+//  }
+//
+//  public void deleteObserver(Observer o) {
+//    if (logger.level <= Logger.FINER) logger.log(this+".deleteObserver("+o+")");
+//    super.deleteObserver(o);
+//    synchronized (obs) {
+//      obs.remove(o);
+//    }
+//  }
+
   /**
    * Method which allows the observers of this socket node handle to be updated.
    * This method sets this object as changed, and then sends out the update.
@@ -320,8 +344,32 @@ public class SocketNodeHandle extends DistNodeHandle {
    * @param update The update
    */
   protected void update(Object update) {
+    if (logger != null) {
+      if (logger.level <= Logger.FINE) {
+//        String s = "";
+//        if (obs != null)
+//          synchronized(obs) {
+//            Iterator i = obs.iterator();
+//            while(i.hasNext())
+//              s+=","+i.next();
+//          }
+        logger.log(this + ".update(" + update + ")" + countObservers());
+      }
+    }
     setChanged();
     notifyObservers(update);
+    if (logger != null) {
+      if (logger.level <= Logger.FINEST) {
+//        String s = "";
+//        if (obs != null)
+//          synchronized(obs) {
+//            Iterator i = obs.iterator();
+//            while(i.hasNext())
+//              s+=","+i.next();
+//          }
+        logger.log(this + ".update(" + update + ")" + countObservers() + " done");
+      }
+    }
   }
 }
 
