@@ -56,7 +56,7 @@ import rice.selector.TimerTask;
  * Class which keeps track of the best routes to remote nodes. This class is
  * also therefore in charge of declaring node death and liveness.
  *
- * @version $Id: SocketSourceRouteManager.java 2901 2006-01-10 15:22:03Z jeffh $
+ * @version $Id: SocketSourceRouteManager.java 3150 2006-03-16 22:17:25Z jeffh $
  * @author Alan Mislove
  */
 public class SocketSourceRouteManager {
@@ -73,6 +73,11 @@ public class SocketSourceRouteManager {
    */
   public long PING_THROTTLE;
 
+  /**
+   * DESCRIBE THE FIELD
+   */
+  public int NUM_SOURCE_ROUTE_ATTEMPTS;
+
   // the local pastry node
   private SocketPastryNode spn;
 
@@ -112,6 +117,7 @@ public class SocketSourceRouteManager {
     Parameters p = node.getEnvironment().getParameters();
     CHECK_DEAD_THROTTLE = p.getLong("pastry_socket_srm_check_dead_throttle");
     PING_THROTTLE = p.getLong("pastry_socket_srm_ping_throttle");
+    NUM_SOURCE_ROUTE_ATTEMPTS = p.getInt("pastry_socket_srm_num_source_route_attempts");
 
     this.logger = node.getEnvironment().getLogManager().getLogger(SocketSourceRouteManager.class, null);
     this.manager = new SocketCollectionManager(node, this, bindAddress, proxyAddress, random);
@@ -241,7 +247,7 @@ public class SocketSourceRouteManager {
    * @return All possible source routes to the destination
    */
   protected SourceRoute[] getAllRoutes(EpochInetSocketAddress destination) {
-    NodeSet nodes = spn.getLeafSet().neighborSet(Integer.MAX_VALUE);
+    NodeSet nodes = spn.getLeafSet().neighborSet(NUM_SOURCE_ROUTE_ATTEMPTS);
     nodes.randomize(spn.getEnvironment().getRandomSource());
     Vector result = new Vector();
     result.add(SourceRoute.build(destination));
@@ -400,8 +406,20 @@ public class SocketSourceRouteManager {
    * @param message The message to send
    * @param address The address to send the message to
    */
-  public void send(EpochInetSocketAddress address, Message message) {
-    getAddressManager(address, true).send(message);
+  public void send(final EpochInetSocketAddress address, final Message message) {
+    if (spn.getEnvironment().getSelectorManager().isSelectorThread()) {
+      getAddressManager(address, true).send(message);
+    } else {
+      if (logger.level <= Logger.FINE) {
+        logger.log("Application attempted to send " + message + " to " + address + " on a non-selector thread.");
+      }
+      spn.getEnvironment().getSelectorManager().invoke(
+        new Runnable() {
+          public void run() {
+            getAddressManager(address, true).send(message);
+          }
+        });
+    }
   }
 
   /**
@@ -493,6 +511,16 @@ public class SocketSourceRouteManager {
     getAddressManager(route.getLastHop(), false).markAlive(route);
   }
 
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param route DESCRIBE THE PARAMETER
+   * @return DESCRIBE THE RETURN VALUE
+   */
+  protected int proximity(SourceRoute route) {
+    return getAddressManager(route.getLastHop(), false).proximity();
+  }
+
   /**
    * This method should be called when a known route is declared suspected.
    *
@@ -911,7 +939,10 @@ public class SocketSourceRouteManager {
       getRouteManager(route).markSuspected();
 
       // mark this address as suspected, if this is currently the best route
-      if ((best != null) && (best.equals(route))) {
+      if (((best == null) || (best.equals(route))) &&
+      // because we set the best == null when we didn't have a route
+      (liveness < SocketNodeHandle.LIVENESS_DEAD)) {
+        // don't promote the node
         setSuspected();
       }
     }
