@@ -42,9 +42,7 @@ import java.util.*;
 
 import rice.environment.logging.Logger;
 import rice.environment.params.Parameters;
-import rice.pastry.NodeHandle;
-import rice.pastry.NodeSetUpdate;
-import rice.pastry.PastryNode;
+import rice.pastry.*;
 import rice.pastry.leafset.LeafSet;
 import rice.pastry.messaging.Address;
 import rice.pastry.messaging.Message;
@@ -65,7 +63,7 @@ import rice.selector.TimerTask;
  * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
  * @author Jeff Hoye
  */
-public class ConsistentJoinProtocol extends StandardJoinProtocol implements Observer, LoopObserver {
+public class ConsistentJoinProtocol extends StandardJoinProtocol implements Observer, NodeSetListener, LoopObserver {
 
   /**
    * This variable is set to prevent the process from going to sleep or not
@@ -85,9 +83,77 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
   WeakHashMap gotResponse;
 
   /**
-   * Nodes that we think are dead.
+   * Nodes that we think are dead. NodeHandle -> FailedTime
    */
-  HashSet failed;
+  Hashtable failed;
+  TimerTask cleanupTask;
+
+//  static class TestNodeHandle extends NodeHandle {
+//    int num;
+//    public TestNodeHandle(int num) {
+//      this.num = num;
+//    }
+//
+//    public NodeId getNodeId() {
+//      return null;
+//    }
+//
+//    public int getLiveness() {
+//      return 0;
+//    }
+//
+//    public int proximity() {
+//      return 0;
+//    }
+//
+//    public boolean ping() {
+//      return false;
+//    }
+//
+//    public boolean equals(Object obj) {
+//      return false;
+//    }
+//
+//    public int hashCode() {
+//      return 0;
+//    }
+//
+//    public void receiveMessage(Message msg) {
+//    }
+//
+//    public String toString() {
+//      return "NH: "+num;
+//    }
+//  }
+//
+//  /**
+//   * To test the comparator...
+//   * @param foo
+//   */
+//  public static void main(String[] foo) {
+//    // we want sort to return newest first, so the list should get flipped
+//    ArrayList l = new ArrayList();
+//    for (int i = 0; i < 20; i++) {
+//      l.add(new FailedTime(new TestNodeHandle(i), i));
+//    }
+//    Collections.sort(l);
+//    Iterator i = l.iterator();
+//    while(i.hasNext()) {
+//      System.out.println(i.next());
+//    }
+//  }
+
+
+  /**
+   * how long a node should remain in failed acquired from
+   * pastry_protocol_consistentJoin_failedRetentionTime
+   */
+  int failedNodeExpirationTime;
+  /**
+   * maximum number of failed entries to gossip sends only the most recent
+   * failures acquired from pastry_protocol_consistentJoin_maxFailedToSend
+   */
+  int maxFailedEntries;
 
   /**
    * NodeHandles I'm observing
@@ -116,14 +182,50 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
                                 PastrySecurityManager sm, RoutingTable rt, LeafSet ls) {
     super(ln, lh, sm, rt, ls);
     gotResponse = new WeakHashMap();
-    failed = new HashSet();
+    failed = new Hashtable();
     observing = new HashSet();
-    ls.addObserver(this);
+    ls.addNodeSetListener(this);
     ln.addObserver(this);
     Parameters p = ln.getEnvironment().getParameters();
     MAX_TIME_TO_BE_SCHEDULED = p.getInt("pastry_protocol_consistentJoin_max_time_to_be_scheduled");
     RETRY_INTERVAL = p.getInt("pastry_protocol_consistentJoin_retry_interval");
+    failedNodeExpirationTime = p.getInt("pastry_protocol_consistentJoin_failedRetentionTime");
+    // 90000
+    maxFailedEntries = p.getInt("pastry_protocol_consistentJoin_maxFailedToSend");
+    // 20
+    int cleanupInterval = p.getInt("pastry_protocol_consistentJoin_cleanup_interval");
+    // 30000
+
     ln.getEnvironment().getSelectorManager().addLoopObserver(this);
+
+    cleanupTask =
+      new TimerTask() {
+        public void run() {
+          if (logger.level <= Logger.INFO) {
+            logger.log("CJP: Cleanup task.");
+          }
+          synchronized (failed) {
+            long now = thePastryNode.getEnvironment().getTimeSource().currentTimeMillis();
+            long expiration = now - failedNodeExpirationTime;
+            Iterator i = failed.values().iterator();
+            while (i.hasNext()) {
+              FailedTime ft = (FailedTime) i.next();
+              if (ft.time < expiration) {
+                if (logger.level <= Logger.FINE) {
+                  logger.log("CJP: Removing " + ft.handle + " from failed set.");
+                }
+                i.remove();
+                ft.handle.deleteObserver(ConsistentJoinProtocol.this);
+              } else {
+                if (logger.level <= Logger.FINER) {
+                  logger.log("CJP: Not Removing " + ft.handle + " from failed set until " + (ft.time + failedNodeExpirationTime) + " which is another " + (ft.time + failedNodeExpirationTime - now) + " millis.");
+                }
+              }
+            }
+          }
+        }
+      };
+    ln.getEnvironment().getSelectorManager().schedule(cleanupTask, cleanupInterval, cleanupInterval);
   }
 
   /**
@@ -139,7 +241,7 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
       logger.log("ChurnJonProtocol.setReady()");
     }
     gotResponse.clear();
-    failed.clear();
+    //failed.clear(); // done by cleanup task as of March 6th, 2006
     // send a probe to everyone in the leafset
     Iterator i = leafSet.neighborSet(Integer.MAX_VALUE).iterator();
     while (i.hasNext()) {
@@ -273,7 +375,7 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
       LeafSet lprime = leafSet.copy();
       for (int i = -cjm.ls.ccwSize(); i <= cjm.ls.cwSize(); i++) {
         NodeHandle nh = cjm.ls.get(i);
-        if (!failed.contains(nh) && nh.getLiveness() < NodeHandle.LIVENESS_DEAD) {
+        if (!failed.containsKey(nh) && nh.getLiveness() < NodeHandle.LIVENESS_DEAD) {
           lprime.put(nh);
         }
       }
@@ -292,7 +394,7 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
       while (it2.hasNext()) {
         NodeHandle nh = (NodeHandle) it2.next();
         // he's not a member, but he could be
-        if (!failed.contains(nh) && nh.getLiveness() < NodeHandle.LIVENESS_DEAD) {
+        if (!failed.containsKey(nh) && nh.getLiveness() < NodeHandle.LIVENESS_DEAD) {
           addToLeafSet(nh);
           // probe
           sendTheMessage(nh, false);
@@ -348,13 +450,13 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
         }
         // failed_i = {}
         //      gotResponse.clear();
-        failed.clear();
-        Iterator it2 = observing.iterator();
-        while (it2.hasNext()) {
-          NodeHandle nh = (NodeHandle) it2.next();
-          nh.deleteObserver(this);
-          it2.remove();
-        }
+        //failed.clear(); // done by cleanup task as of March 6th, 2006
+//        Iterator it2 = observing.iterator();
+//        while(it2.hasNext()) {
+//          NodeHandle nh = (NodeHandle)it2.next();
+//          nh.deleteObserver(this);
+//          it2.remove();
+//        }
       } else {
         if (logger.level <= Logger.FINE) {
           logger.log("CJP: still need to hear from:" + toHearFromStr);
@@ -364,9 +466,27 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
       if (logger.level <= Logger.FINE) {
         logger.log("CJP: LS is not complete: " + leafSet);
       }
-      // need to poll left and right neighbors
       // sendTheMessage to leftmost and rightmost?
-      // send leafsetMaintenance to self?
+
+      NodeHandle left = null;
+      NodeHandle right = null;
+
+      synchronized (leafSet) {
+        int index = -leafSet.ccwSize();
+        if (index != -leafSet.maxSize() / 2) {
+          left = leafSet.get(index);
+        }
+        index = leafSet.cwSize();
+        if (index != leafSet.maxSize() / 2) {
+          right = leafSet.get(index);
+        }
+      }
+      if (left != null) {
+        sendTheMessage(left, true);
+      }
+      if (right != null) {
+        sendTheMessage(right, true);
+      }
     }
   }
 
@@ -374,7 +494,8 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
    * Sends a consistent join protocol message.
    *
    * @param nh
-   * @param reply
+   * @param reply if the reason we are sending this message is just as a
+   *      response
    */
   public void sendTheMessage(NodeHandle nh, boolean reply) {
     if (!reply) {
@@ -389,10 +510,44 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
 
     // todo, may want to repeat this message as long as the node is alive if we
     // are worried about rare message drops
-    if (thePastryNode.isReady()) {
-      failed.clear();
+//    if (thePastryNode.isReady()) {
+    //failed.clear(); // done by cleanup task as of March 6th, 2006
+//    }
+    HashSet toSend;
+    if (failed.size() < maxFailedEntries) {
+      toSend = new HashSet(failed.keySet());
+    } else {
+      ArrayList l = new ArrayList(failed.values());
+      Collections.sort(l);
+      toSend = new HashSet();
+      for (int i = 0; i < maxFailedEntries; i++) {
+        FailedTime tf = (FailedTime) l.get(i);
+        toSend.add(tf.handle);
+      }
     }
-    nh.receiveMessage(new ConsistentJoinMsg(getAddress(), leafSet, failed, !reply));
+    nh.receiveMessage(new ConsistentJoinMsg(getAddress(), leafSet, toSend, !reply));
+  }
+
+  /**
+   * DESCRIBE THE METHOD
+   *
+   * @param set DESCRIBE THE PARAMETER
+   * @param handle DESCRIBE THE PARAMETER
+   * @param added DESCRIBE THE PARAMETER
+   */
+  public void nodeSetUpdate(NodeSetEventSource set, NodeHandle handle, boolean added) {
+    if (thePastryNode.isReady()) {
+      return;
+    }
+    if (added) {
+      if (gotResponse.get(handle) == null) {
+        sendTheMessage(handle, false);
+      }
+    } else {
+      doneProbing();
+    }
+
+    return;
   }
 
   /**
@@ -402,6 +557,9 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
    * @param arg DESCRIBE THE PARAMETER
    */
   public void update(Observable arg0, Object arg) {
+    if (logger.level <= Logger.FINEST) {
+      logger.log("CJP: update(" + arg0 + "," + arg + ")" + arg.getClass().getName());
+    }
 
     // we went offline for whatever reason.  Now we need to try to come back online.
     if (arg0 == thePastryNode) {
@@ -410,28 +568,12 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
       }
     }
 
-    if (arg instanceof NodeSetUpdate) {
-      if (thePastryNode.isReady()) {
-        return;
-      }
-      NodeSetUpdate nsu = (NodeSetUpdate) arg;
-      if (nsu.wasAdded()) {
-        if (gotResponse.get(nsu.handle()) == null) {
-          sendTheMessage(nsu.handle(), false);
-        }
-      } else {
-        doneProbing();
-      }
-
-      return;
-    }
-
     if (arg0 instanceof NodeHandle) {
-      if (thePastryNode.isReady()) {
-        observing.remove(arg0);
-        arg0.deleteObserver(this);
-        return;
-      }
+//      if (thePastryNode.isReady()) {
+//        observing.remove(arg0);
+//        arg0.deleteObserver(this);
+//        return;
+//      }
 
       // assume it's a NodeHandle, cause we
       // want to throw the exception if it is something we don't recognize
@@ -440,7 +582,9 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
         if (logger.level <= Logger.FINE) {
           logger.log("CJP:" + arg0 + " declared dead");
         }
-        failed.add(nh);
+        if (!failed.containsKey(nh)) {
+          failed.put(nh, new FailedTime(nh, thePastryNode.getEnvironment().getTimeSource().currentTimeMillis()));
+        }
         leafSet.remove(nh);
         doneProbing();
       }
@@ -488,6 +632,7 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
       logger.log("CJP: destroy() called");
     }
     thePastryNode.getEnvironment().getSelectorManager().removeLoopObserver(this);
+    cleanupTask.cancel();
   }
 
   /**
@@ -506,4 +651,48 @@ public class ConsistentJoinProtocol extends StandardJoinProtocol implements Obse
       super(dest);
     }
   }
+  // cleans up failed
+
+  /**
+   * DESCRIBE THE CLASS
+   *
+   * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
+   * @author jeffh
+   */
+  static class FailedTime implements Comparable {
+    long time;
+    NodeHandle handle;
+
+    /**
+     * Constructor for FailedTime.
+     *
+     * @param handle DESCRIBE THE PARAMETER
+     * @param time DESCRIBE THE PARAMETER
+     */
+    public FailedTime(NodeHandle handle, long time) {
+      this.time = time;
+      this.handle = handle;
+    }
+
+    /**
+     * DESCRIBE THE METHOD
+     *
+     * @param arg0 DESCRIBE THE PARAMETER
+     * @return DESCRIBE THE RETURN VALUE
+     */
+    public int compareTo(Object arg0) {
+      FailedTime ft = (FailedTime) arg0;
+      // note this is backwards, because we want them sorted in reverse order
+      return (int) (ft.time - this.time);
+    }
+
+    /**
+     * DESCRIBE THE METHOD
+     *
+     * @return DESCRIBE THE RETURN VALUE
+     */
+    public String toString() {
+      return "FT:" + handle + " " + time;
+    }
+  }
 }
