@@ -49,7 +49,7 @@ import java.util.*;
 /**
  * An implementation of a simple join protocol.
  *
- * @version $Id: StandardJoinProtocol.java 2902 2006-01-10 15:23:05Z jeffh $
+ * @version $Id: StandardJoinProtocol.java 3095 2006-02-23 20:14:55Z jstewart $
  * @author Peter Druschel
  * @author Andrew Ladd
  * @author Rongmei Zhang
@@ -199,18 +199,20 @@ public class StandardJoinProtocol extends PastryAppl {
 
       jh = security.verifyNodeHandle(jh);
 
-      int base = thePastryNode.getRoutingTable().baseBitLength();
+      if (!jh.equals(localHandle)) {
+        int base = thePastryNode.getRoutingTable().baseBitLength();
 
-      int msdd = localId.indexOfMSDD(nid, base);
-      int last = jr.lastRow();
+        int msdd = localId.indexOfMSDD(nid, base);
+        int last = jr.lastRow();
 
-      for (int i = last - 1; msdd > 0 && i >= msdd; i--) {
-        RouteSet row[] = routeTable.getRow(i);
+        for (int i = last - 1; msdd > 0 && i >= msdd; i--) {
+          RouteSet[] row = routeTable.getRow(i);
 
-        jr.pushRow(row);
+          jr.pushRow(row);
+        }
+
+        rm.routeMessage(localHandle);
       }
-
-      rm.routeMessage(localHandle);
     } else if (msg instanceof InitiateJoin) {
       // request from the local node to
       // join
@@ -218,20 +220,21 @@ public class StandardJoinProtocol extends PastryAppl {
 
       NodeHandle nh = ij.getHandle();
 
-      nh = security.verifyNodeHandle(nh);
-
       if (nh == null) {
         if (logger.level <= Logger.SEVERE) {
           logger.log(
             "ERROR: Cannot join ring.  All bootstraps are faulty.");
         }
-      } else if (nh.isAlive() == true) {
-        JoinRequest jr = new JoinRequest(localHandle, thePastryNode.getRoutingTable().baseBitLength());
+      } else {
+        nh = security.verifyNodeHandle(nh);
+        if (nh.isAlive() == true) {
+          JoinRequest jr = new JoinRequest(localHandle, thePastryNode.getRoutingTable().baseBitLength());
 
-        RouteMessage rm = new RouteMessage(localHandle.getNodeId(), jr,
-          new PermissiveCredentials(), getAddress());
-        rm.getOptions().setRerouteIfSuspected(false);
-        nh.bootstrap(rm);
+          RouteMessage rm = new RouteMessage(localHandle.getNodeId(), jr,
+            new PermissiveCredentials(), getAddress());
+          rm.getOptions().setRerouteIfSuspected(false);
+          nh.bootstrap(rm);
+        }
       }
     }
   }
