@@ -39,6 +39,7 @@ package rice.pastry.testing;
 import rice.environment.Environment;
 import rice.environment.logging.Logger;
 import rice.environment.params.simple.SimpleParameters;
+import rice.environment.time.simulated.DirectTimeSource;
 import rice.pastry.*;
 import rice.pastry.direct.*;
 import rice.pastry.standard.*;
@@ -59,7 +60,7 @@ import java.util.*;
  * Potentially this test should be run daily and the proximity recorded over
  * time to see if there was a drastic change based on algorithmic change.
  *
- * @version $Id: ClosestRegrTest.java 2844 2005-12-15 13:07:05Z jeffh $
+ * @version $Id: ClosestRegrTest.java 3038 2006-02-07 10:01:01Z jeffh $
  * @author alan mislove
  */
 public class ClosestRegrTest {
@@ -84,8 +85,7 @@ public class ClosestRegrTest {
    * @exception IOException DESCRIBE THE EXCEPTION
    */
   private ClosestRegrTest() throws IOException {
-    environment = new Environment(null, null, null, new DirectTimeSource(System.currentTimeMillis()), null,
-      new SimpleParameters(Environment.defaultParamFileArray, null));
+    environment = Environment.directEnvironment();
     simulator = new SphereNetwork(environment);
     factory = new DirectPastryNodeFactory(new RandomNodeIdFactory(environment), simulator, environment);
     pastryNodes = new Vector();
@@ -130,12 +130,20 @@ public class ClosestRegrTest {
   protected void run() {
     for (int i = 0; i < NUM_NODES; i++) {
       PastryNode node = factory.newNode(getBootstrap());
+      synchronized (node) {
+        while (!node.isReady()) {
+          try {
+            node.wait(500);
+          } catch (InterruptedException ie) {
+            return;
+          }
+        }
+      }
       if (i > 0) {
         test(i, (DirectNodeHandle) node.getLocalHandle());
       }
 
-      while (simulator.simulate()) {
-      }
+//      while (simulator.simulate()) {}
 
       System.out.println("CREATED NODE " + i + " " + node.getNodeId());
 
