@@ -37,6 +37,7 @@ package rice.pastry.testing;
 
 import rice.environment.Environment;
 import rice.environment.params.simple.SimpleParameters;
+import rice.environment.time.simulated.DirectTimeSource;
 import rice.pastry.*;
 import rice.pastry.direct.*;
 import rice.pastry.standard.*;
@@ -53,7 +54,7 @@ import java.util.*;
 /**
  * PastryRegrTest a regression test suite for pastry.
  *
- * @version $Id: DirectPastryRegrTest.java 2896 2006-01-09 17:11:50Z jeffh $
+ * @version $Id: DirectPastryRegrTest.java 3048 2006-02-10 16:25:33Z jeffh $
  * @author andrew ladd
  * @author peter druschel
  * @author sitaram iyer
@@ -68,8 +69,7 @@ public class DirectPastryRegrTest extends PastryRegrTest {
    * @exception IOException DESCRIBE THE EXCEPTION
    */
   private DirectPastryRegrTest() throws IOException {
-    super(new Environment(null, null, null, new DirectTimeSource(System.currentTimeMillis()), null,
-      new SimpleParameters(Environment.defaultParamFileArray, null)));
+    super(Environment.directEnvironment());
     simulator = new SphereNetwork(environment);
     factory = new DirectPastryNodeFactory(new RandomNodeIdFactory(environment),
       simulator,
@@ -118,11 +118,15 @@ public class DirectPastryRegrTest extends PastryRegrTest {
    * @return DESCRIBE THE RETURN VALUE
    */
   protected boolean simulate() {
-    boolean res = simulator.simulate();
-    if (res) {
-      msgCount++;
+    try {
+      Thread.sleep(300);
+    } catch (InterruptedException ie) {
     }
-    return res;
+    return false;
+//    boolean res = simulator.simulate();
+//    if (res)
+//      msgCount++;
+//    return res;
   }
 
   // do nothing in the simulated world
@@ -153,7 +157,7 @@ public class DirectPastryRegrTest extends PastryRegrTest {
    */
   public static void main(String args[]) throws IOException {
     DirectPastryRegrTest pt = new DirectPastryRegrTest();
-    mainfunc(pt, args, 1000
+    mainfunc(pt, args, 500
     /*
      *  n
      */
