@@ -35,7 +35,10 @@ if advised of the possibility of such damage.
 ********************************************************************************/
 package rice.pastry.testing;
 
+import rice.Continuation;
+import rice.Continuation.*;
 import rice.environment.Environment;
+import rice.p2p.commonapi.RangeCannotBeDeterminedException;
 import rice.pastry.*;
 import rice.pastry.direct.*;
 import rice.pastry.standard.*;
@@ -51,7 +54,7 @@ import java.util.*;
 /**
  * PastryRegrTest a regression test suite for pastry. abstract class.
  *
- * @version $Id: PastryRegrTest.java 2896 2006-01-09 17:11:50Z jeffh $
+ * @version $Id: PastryRegrTest.java 3249 2006-04-27 13:08:32Z jeffh $
  * @author andrew ladd
  * @author peter druschel
  */
@@ -71,6 +74,10 @@ public abstract class PastryRegrTest {
    * of NodeId
    */
   public SortedMap pastryNodesSorted;
+  /**
+   * DESCRIBE THE FIELD
+   */
+  public SortedMap pastryNodesSortedReady;
 
   /**
    * DESCRIBE THE FIELD
@@ -99,8 +106,6 @@ public abstract class PastryRegrTest {
    */
   public NodeId lastNode;
 
-  int msgCount = 0;
-
   /**
    * DESCRIBE THE FIELD
    */
@@ -115,11 +120,14 @@ public abstract class PastryRegrTest {
     this.environment = env;
     pastryNodes = new Vector();
     pastryNodesSorted = new TreeMap();
+    pastryNodesSortedReady = new TreeMap();
     pastryNodesLastAdded = new Vector();
     inConcJoin = false;
     rtApps = new Vector();
   }
 
+//  int msgCount = 0;
+
   // abstract methods
 
   /**
@@ -179,13 +187,11 @@ public abstract class PastryRegrTest {
 
     registerapp(pn, rta);
 
-    int msgCount = 0;
+//    int msgCount = 0;
 
-    if (bootstrap != null) {
-      while (simulate()) {
-        msgCount++;
-      }
-    }
+//    if (bootstrap != null)
+//      while (simulate())
+//        msgCount++;
 
     //System.out.println("created " + pn + " messages: " + msgCount);
 
@@ -244,31 +250,39 @@ public abstract class PastryRegrTest {
         if (n == 0) {
           // we have to join the first batch of nodes
           // sequentially, else we create multiple rings
-          while (simulate()) {
-            msgCount++;
-          }
+//          while (simulate())
+//            msgCount++;
 
           // ADDED FOR WIRE PROTOCOL...
-          while (!pn.isReady()) {
-            pause(500);
+          synchronized (pn) {
+            while (!pn.isReady()) {
+              try {
+                pn.wait(500);
+              } catch (InterruptedException ie) {
+              }
+            }
           }
         }
       }
     }
 
-    int msgCount = 0;
+//    int msgCount = 0;
 
     // now simulate concurrent joins
-    while (simulate()) {
-      msgCount++;
-    }
+//    while (simulate())
+//      msgCount++;
 
     // ADDED FOR WIRE PROTOCOL...
     // wait until all nodes are ready
     for (int i = 0; i < pastryNodesLastAdded.size(); i++) {
       PastryNode pn = (PastryNode) pastryNodesLastAdded.get(i);
-      while (!pn.isReady()) {
-        pause(500);
+      synchronized (pn) {
+        while (!pn.isReady()) {
+          try {
+            pn.wait(500);
+          } catch (InterruptedException ie) {
+          }
+        }
       }
     }
     pause(2500);
@@ -277,12 +291,35 @@ public abstract class PastryRegrTest {
 
     for (int i = 0; i < num; i++) {
       System.out.println("created " + rta[i].getNodeId());
+      PastryNode pn = rta[i].getPastryNode();
+      pastryNodesSortedReady.put(pn.getNodeId(), pn);
 
       checkLeafSet(rta[i]);
-      checkRoutingTable(rta[i]);
+      if (!environment.getSelectorManager().isSelectorThread() && (this instanceof DirectPastryRegrTest)) {
+        final RegrTestApp theRta = rta[i];
+        final DirectPastryNode dpn = (DirectPastryNode) theRta.getPastryNode();
+
+        // check if closest entry has valid proximity
+        ExternalRunnable er =
+          new ExternalRunnable() {
+            protected Object execute() throws Exception {
+              DirectPastryNode.currentNode = dpn;
+              checkRoutingTable(theRta);
+              DirectPastryNode.currentNode = null;
+              return null;
+            }
+          };
+        try {
+          er.invoke(environment);
+        } catch (Exception e) {
+          e.printStackTrace();
+        }
+      } else {
+        checkRoutingTable(rta[i]);
+      }
     }
 
-    System.out.println("messages: " + msgCount);
+//    System.out.println("messages: " + msgCount);
 
     for (int i = 0; i < rtApps.size(); i++) {
       //checkLeafSet((RegrTestApp)rtApps.get(i));
@@ -323,15 +360,11 @@ public abstract class PastryRegrTest {
 
       // send to a random node
       rta.sendTrace(pn.getNodeId());
-      while (simulate()) {
-        ;
-      }
+//      while (simulate());
 
       // send to a random key
       rta.sendTrace(key);
-      while (simulate()) {
-        ;
-      }
+//      while (simulate());
 
       //System.out.println("-------------------");
     }
@@ -499,6 +532,7 @@ public abstract class PastryRegrTest {
           try {
             range = rta.range(ls.get(k), maxRank, nearest, true);
           } catch (IllegalArgumentException iae) {
+          } catch (RangeCannotBeDeterminedException rcbde) {
           }
         } else {
           // check the individual rank ranges
@@ -587,7 +621,7 @@ public abstract class PastryRegrTest {
    * @param rta DESCRIBE THE PARAMETER
    */
 
-  private void checkRoutingTable(RegrTestApp rta) {
+  private void checkRoutingTable(final RegrTestApp rta) {
     RoutingTable rt = rta.getRoutingTable();
 
     // check routing table
@@ -600,7 +634,7 @@ public abstract class PastryRegrTest {
         // skip if local nodeId digit
         // if (j == rta.getNodeId().getDigit(i,rt.baseBitLength())) continue;
 
-        RouteSet rs = rt.getRouteSet(i, j);
+        final RouteSet rs = rt.getRouteSet(i, j);
 
         Id domainFirst = rta.getNodeId().getDomainPrefix(i, j, 0,
           rt.baseBitLength());
@@ -628,12 +662,32 @@ public abstract class PastryRegrTest {
           }
         } else {
           // check entries
+          NodeHandle nh;
+          if (!environment.getSelectorManager().isSelectorThread() && (this instanceof DirectPastryRegrTest)) {
+            // check if closest entry has valid proximity
+            ExternalRunnable er =
+              new ExternalRunnable() {
+                protected Object execute() throws Exception {
+                  DirectPastryNode temp = DirectPastryNode.currentNode;
+                  DirectPastryNode.currentNode = (DirectPastryNode) rta.getPastryNode();
+                  Object ret = rs.closestNode();
+                  DirectPastryNode.currentNode = temp;
+                  return ret;
+                }
+              };
+            try {
+              nh = (NodeHandle) er.invoke(environment);
+            } catch (Exception e) {
+              e.printStackTrace();
+              System.exit(0);
+              return;
+            }
+          } else {
+            nh = (NodeHandle) rs.closestNode();
+          }
+
 
-          // check if closest entry has valid proximity
-          DirectPastryNode.currentNode = (DirectPastryNode) rta.getPastryNode();
-          NodeHandle nh = rs.closestNode();
           int bestProximity = Integer.MAX_VALUE;
-          ;
           if (nh != null) {
             bestProximity = nh.proximity();
             if (nh.proximity() == Integer.MAX_VALUE) {
@@ -756,8 +810,9 @@ public abstract class PastryRegrTest {
         //if ((i + numConcJoins) % m == 0) {
         Date now = new Date();
         System.out.println(pt.pastryNodes.size() + " "
-          + (now.getTime() - old.getTime()) + " " + pt.msgCount);
-        pt.msgCount = 0;
+          + (now.getTime() - old.getTime()));
+        // + " " + pt.msgCount);
+//        pt.msgCount = 0;
         old = now;
       }
 
