@@ -42,7 +42,7 @@ package rice.persistence;
  *  @author Ansley Post
  *  @author Alan Mislove
  *
- *  @version $Id: PersistentStorage.java 2873 2005-12-21 19:00:54Z jstewart $
+ *  @version $Id: PersistentStorage.java 3007 2006-02-02 13:50:02Z amislove $
  */
 import java.io.*;
 import java.util.*;
@@ -65,17 +65,19 @@ import rice.p2p.util.*;
  * toString() method returns a String of constant length. It has been exteneded
  * to support variable-length toString()s, but we have the caveat that not
  * toString() can be a substring of another toString() - this will cause
- * undefined behavior. The serialized objects are stored on-disk in a GZIPed XML
- * format, which provides extensibility with reasonable storage and processing
- * costs. Additionally, any metadata, if provided, is also stored in the on-disk
- * file. The format of the file is [Key, Object, Version, Gzipped XML]
- * [Metadata, Gzipped XML] [persistence magic number, long] [persistence
- * version, long] [persistence revision, long] [metadata length, long] The
- * persistence package is set up to automatically upgrade older versions of the
- * on-disk format as new data is written under the key. Persistence also
- * supports the metadata interface specified in the Catalog interface. All
- * metadata is guaranteeded to be stored in memory, so fetching the metadata of
- * a given key is an efficient operation.
+ * undefined behavior. <b>Additionally, the toString() method on the key Ids
+ * *CANNOT* have the period ('.') or exclamation point ('!') characters in them
+ * - these are used for internal purposes.<b> The serialized objects are stored
+ * on-disk in a GZIPed XML format, which provides extensibility with reasonable
+ * storage and processing costs. Additionally, any metadata, if provided, is
+ * also stored in the on-disk file. The format of the file is [Key, Object,
+ * Version, Gzipped XML] [Metadata, Gzipped XML] [persistence magic number,
+ * long] [persistence version, long] [persistence revision, long] [metadata
+ * length, long] The persistence package is set up to automatically upgrade
+ * older versions of the on-disk format as new data is written under the key.
+ * Persistence also supports the metadata interface specified in the Catalog
+ * interface. All metadata is guaranteeded to be stored in memory, so fetching
+ * the metadata of a given key is an efficient operation.
  *
  * @version $Id: pretty.settings 2305 2005-03-11 20:22:33Z jeffh $
  * @author jeffh
@@ -176,6 +178,11 @@ public class PersistentStorage implements Storage {
    */
   public final static int METADATA_SYNC_TIME = 300000;
 
+  /**
+   * Special placeholder for the file whose name should be zero-length
+   */
+  public final static String ZERO_LENGTH_NAME = "!";
+
   /**
    * Builds a PersistentStorage given a root directory in which to persist the
    * data. Uses a default instance name.
@@ -295,7 +302,7 @@ public class PersistentStorage implements Storage {
               }
 
               if (logger.level <= Logger.FINER) {
-                logger.log("COUNT: Fetching data under " + id + " of size " + objFile.length() + " in " + name);
+                logger.log("COUNT: Fetching data under " + id.toStringFull() + " of size " + objFile.length() + " in " + name);
               }
               return readData(objFile);
             } catch (Exception e) {
@@ -379,7 +386,9 @@ public class PersistentStorage implements Storage {
     CharacterHashSet set = new CharacterHashSet();
 
     for (int i = 0; i < names.length; i++) {
-      set.put(names[i].charAt(length));
+      if (names[i].length() > length) {
+        set.put(names[i].charAt(length));
+      }
     }
 
     char[] splits = set.get();
@@ -400,7 +409,7 @@ public class PersistentStorage implements Storage {
    * @return The longest common prefix of all of the names
    */
   private int getPrefixLength(String[] names) {
-    int length = names[0].length();
+    int length = names[0].length() - 1;
 
     for (int i = 0; i < names.length; i++) {
       length = getPrefixLength(names[0], names[i], length);
@@ -421,7 +430,7 @@ public class PersistentStorage implements Storage {
   private int getPrefixLength(String a, String b, int max) {
     int i = 0;
 
-    for (; (i < a.length()) && (i < b.length()) && (i < max); i++) {
+    for (; (i < a.length() - 1) && (i < b.length() - 1) && (i < max); i++) {
       if (a.charAt(i) != b.charAt(i)) {
         return i;
       }
@@ -464,7 +473,18 @@ public class PersistentStorage implements Storage {
    */
   private File getFile(Id id) throws IOException {
     File dir = getDirectoryForId(id);
-    return new File(dir, id.toStringFull().substring(getPrefix(dir).length()));
+    String name = id.toStringFull().substring(getPrefix(dir).length());
+    if (name.equals("")) {
+      name = ZERO_LENGTH_NAME;
+    }
+
+    // check for ! directory
+    File file = new File(dir, name);
+    if (file.exists() && file.isDirectory()) {
+      file = new File(file, ZERO_LENGTH_NAME);
+    }
+
+    return file;
   }
 
   /**
@@ -506,14 +526,16 @@ public class PersistentStorage implements Storage {
       for (int i = 0; i < subDirs.length; i++) {
         if (name.startsWith(subDirs[i].getName())) {
           return getDirectoryForName(name.substring(subDirs[i].getName().length()), subDirs[i]);
+        } else if ((name.length() == 0) && subDirs[i].getName().equals(ZERO_LENGTH_NAME)) {
+          return getDirectoryForName(name, subDirs[i]);
         }
       }
 
       /*
        *  here, we must create the appropriate directory
        */
-      if (name.length() >= subDirs[0].getName().length()) {
-        File newDir = new File(dir, name.substring(0, subDirs[0].getName().length()));
+      if ((name.length() >= subDirs[0].getName().length()) || ((name.length() == 0) && (subDirs[0].getName().length() == 1))) {
+        File newDir = new File(dir, (name.length() == 0 ? ZERO_LENGTH_NAME : name.substring(0, subDirs[0].getName().length())));
         if (logger.level <= Logger.FINE) {
           logger.log("Necessarily creating dir " + newDir.getName());
         }
@@ -541,7 +563,7 @@ public class PersistentStorage implements Storage {
         for (int i = 0; i < subDirs.length; i++) {
           dirs[i] = subDirs[i].getName();
         }
-        dirs[subDirs.length] = name;
+        dirs[subDirs.length] = (name.length() == 0 ? ZERO_LENGTH_NAME : name);
 
         /*
          *  now reformat the directory, creating an entry for this name
@@ -580,7 +602,7 @@ public class PersistentStorage implements Storage {
 
     StringBuffer buffer = new StringBuffer();
     while (!file.equals(appDirectory)) {
-      buffer.insert(0, file.getName());
+      buffer.insert(0, file.getName().replaceAll(ZERO_LENGTH_NAME, ""));
       file = file.getParentFile();
     }
     prefixes.put(file, buffer.toString());
@@ -882,6 +904,11 @@ public class PersistentStorage implements Storage {
    * @param c The command to run once the operation is complete
    */
   public void store(final Id id, final Serializable metadata, final Serializable obj, Continuation c) {
+    if (id == null || obj == null) {
+      c.receiveResult(new Boolean(false));
+      return;
+    }
+
     printStats();
 
     environment.getProcessor().processBlockingIO(
@@ -896,7 +923,7 @@ public class PersistentStorage implements Storage {
           }
 
           if (logger.level <= Logger.FINER) {
-            logger.log("Storing object " + obj + " under id " + id + " in root " + appDirectory);
+            logger.log("Storing object " + obj + " under id " + id.toStringFull() + " in root " + appDirectory);
           }
 
           /*
@@ -905,13 +932,17 @@ public class PersistentStorage implements Storage {
           File objFile = getFile(id);
           File transcFile = makeTemporaryFile(id);
 
+          if (logger.level <= Logger.FINER) {
+            logger.log("Writing object " + obj + " to temporary file " + transcFile + " and renaming to " + objFile);
+          }
+
           /*
            *  next, write out the data to a new copy of the original file
            */
           try {
             writeObject(obj, metadata, id, environment.getTimeSource().currentTimeMillis(), transcFile);
             if (logger.level <= Logger.FINER) {
-              logger.log("Done writing object " + obj + " under id " + id + " in root " + appDirectory);
+              logger.log("Done writing object " + obj + " under id " + id.toStringFull() + " in root " + appDirectory);
             }
 
             /*
@@ -1280,8 +1311,6 @@ public class PersistentStorage implements Storage {
            */
           if (dirs.length > 0) {
             moveFileToCorrectDirectory(dir, files[i]);
-          } else {
-            upgradeFile(dir, files[i]);
           }
         }
       } catch (Exception e) {
@@ -1330,23 +1359,6 @@ public class PersistentStorage implements Storage {
     return true;
   }
 
-  /**
-   * Method which checks to see if the given file needs to be upgraded to a new
-   * version of the persistent storage layout.
-   *
-   * @param parent The parent directory
-   * @param name The name of the file
-   * @exception IOException DESCRIBE THE EXCEPTION
-   */
-  private void upgradeFile(File parent, String name) throws IOException {
-    if (name.startsWith(getPrefix(parent)) && (!parent.equals(appDirectory))) {
-      if (logger.level <= Logger.FINE) {
-        logger.log("Upgrading file " + name + " to new version " + name.substring(getPrefix(parent).length()));
-      }
-      renameFile(new File(parent, name), new File(parent, name.substring(getPrefix(parent).length())));
-    }
-  }
-
   /**
    * Inititializes the idSet data structure In doing this it must resolve
    * conflicts and aborted transactions. After this is run the most current
@@ -1414,7 +1426,7 @@ public class PersistentStorage implements Storage {
            */
           if (index && ((!metadata.containsKey(id)) || (files[i].lastModified() > modified))) {
             if (logger.level <= Logger.FINER) {
-              logger.log("Reading newer metadata out of file " + files[i] + " id " + id + " " + files[i].lastModified() + " " + modified + " " + metadata.containsKey(id));
+              logger.log("Reading newer metadata out of file " + files[i] + " id " + id.toStringFull() + " " + files[i].lastModified() + " " + modified + " " + metadata.containsKey(id));
             }
             metadata.put(id, readMetadata(files[i]));
             dirty.add(dir);
@@ -1436,9 +1448,17 @@ public class PersistentStorage implements Storage {
       }
     }
 
+    /*
+     *  now recurse and check all of the children
+     */
     for (int i = 0; i < dirs.length; i++) {
       initFileMap(dirs[i]);
     }
+
+    /*
+     *  and finally see if this directory needs to be pruned or expanded
+     */
+    checkDirectory(dir);
   }
 
   /**
@@ -1571,7 +1591,7 @@ public class PersistentStorage implements Storage {
       logger.log("Expanding directory " + dir + " due to too many subdirectories");
     }
     /*
-     *  first, determine what directories we should create
+     *  first, determine what directories we should create (ignoring the ! directories)
      */
     String[] newDirNames = getDirectories(dir.list(new DirectoryFilter()));
     reformatDirectory(dir, newDirNames);
@@ -1599,7 +1619,7 @@ public class PersistentStorage implements Storage {
       newDirs[i] = new File(dir, newDirNames[i]);
       createDirectory(newDirs[i]);
       if (logger.level <= Logger.FINE) {
-        logger.log("creating directory " + newDirNames[i]);
+        logger.log("Creating directory " + newDirNames[i]);
       }
 
       /*
@@ -1615,7 +1635,7 @@ public class PersistentStorage implements Storage {
         File oldDir = new File(dir, subDirNames[j]);
         newSubDirs[j] = new File(newDirs[i], subDirNames[j].substring(newDirNames[i].length()));
         if (logger.level <= Logger.FINE) {
-          logger.log("moving the old direcotry " + oldDir + " to " + newSubDirs[j]);
+          logger.log("Moving the old direcotry " + oldDir + " to " + newSubDirs[j]);
         }
         renameFile(oldDir, newSubDirs[j]);
 
@@ -1661,9 +1681,14 @@ public class PersistentStorage implements Storage {
     for (int i = 0; i < dirNames.length; i++) {
       dirs[i] = new File(dir, dirNames[i]);
       directories.put(dirs[i], new File[0]);
+
+      if (dirs[i].exists() && dirs[i].isFile()) {
+        renameFile(dirs[i], new File(dir, dirs[i].getName() + ZERO_LENGTH_NAME));
+      }
+
       createDirectory(dirs[i]);
       if (logger.level <= Logger.FINE) {
-        logger.log("creating directory " + dirNames[i]);
+        logger.log("Creating directory " + dirNames[i]);
       }
 
       /*
@@ -1686,6 +1711,9 @@ public class PersistentStorage implements Storage {
     for (int i = 0; i < files.length; i++) {
       for (int j = 0; j < dirs.length; j++) {
         if (files[i].getName().startsWith(dirs[j].getName())) {
+          if (logger.level <= Logger.FINEST) {
+            logger.log("Renaming file " + files[i] + " to " + new File(dirs[j], files[i].getName().substring(dirs[j].getName().length())));
+          }
           renameFile(files[i], new File(dirs[j], files[i].getName().substring(dirs[j].getName().length())));
           break;
         }
@@ -1714,7 +1742,7 @@ public class PersistentStorage implements Storage {
   private void moveFileToCorrectDirectory(File parent, String name) throws IOException {
     File file = new File(parent, name);
     Id id = readKeyFromFile(file);
-    File dest = getDirectoryForName(id.toStringFull());
+    File dest = getDirectoryForId(id);
 
     /*
      *  if it's in the wrong directory, then move it and resolve the conflict if necessary
@@ -2036,7 +2064,7 @@ public class PersistentStorage implements Storage {
    * @return The key that was read in
    */
   private Id readKey(File file) {
-    String s = getPrefix(file.getParentFile()) + file.getName();
+    String s = getPrefix(file.getParentFile()) + file.getName().replaceAll(ZERO_LENGTH_NAME, "");
 
     if (s.indexOf(".") >= 0) {
       return factory.buildIdFromToString(s.toCharArray(), 0, s.indexOf("."));
@@ -2103,7 +2131,7 @@ public class PersistentStorage implements Storage {
    * @exception IOException DESCRIBE THE EXCEPTION
    */
   private static void createDirectory(File directory) throws IOException {
-    if ((directory != null) && (!directory.exists()) && (!directory.mkdir())) {
+    if ((directory == null) || (directory.exists() && directory.isFile()) || (!(directory.exists()) && (!directory.mkdir()))) {
       throw new IOException("Creation of directory " + directory + " failed!");
     }
   }
